# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: gcsdk_gcmessages.proto
# plugin: python-betterproto

from dataclasses import dataclass

import betterproto

from ....protobufs.msg import GCProtobufMessage
from ..enums import EMsg


class GcClientLauncherType(betterproto.Enum):
    Default = 0
    PerfectWorld = 1
    SteamChina = 2


class GcConnectionStatus(betterproto.Enum):
    HaveSession = 0
    GCGoingDown = 1
    NoSession = 2
    NoSessionInLogonQueue = 3
    NoSteam = 4


@dataclass(eq=False, repr=False)
class IDOwner(betterproto.Message):
    type: int = betterproto.uint32_field(1)
    id: int = betterproto.uint64_field(2)


class SOCreate(GCProtobufMessage, msg=EMsg.SOCreate):
    type_id: int = betterproto.int32_field(2)
    object_data: bytes = betterproto.bytes_field(3)
    version: int = betterproto.fixed64_field(4)
    owner_soid: "IDOwner" = betterproto.message_field(5)


class SOUpdate(GCProtobufMessage, msg=EMsg.SOUpdate):
    type_id: int = betterproto.int32_field(2)
    object_data: bytes = betterproto.bytes_field(3)
    version: int = betterproto.fixed64_field(4)
    owner_soid: "IDOwner" = betterproto.message_field(5)


class SODestroy(GCProtobufMessage, msg=EMsg.SODestroy):
    type_id: int = betterproto.int32_field(2)
    object_data: bytes = betterproto.bytes_field(3)
    version: int = betterproto.fixed64_field(4)
    owner_soid: "IDOwner" = betterproto.message_field(5)


class MultipleObjects(GCProtobufMessage, msg=EMsg.SOUpdateMultiple):
    objects_modified: "list[MultipleObjectsSingleObject]" = betterproto.message_field(2)
    version: int = betterproto.fixed64_field(3)
    owner_soid: "IDOwner" = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class MultipleObjectsSingleObject(betterproto.Message):
    type_id: int = betterproto.int32_field(1)
    object_data: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class CacheSubscribed(betterproto.Message):
    objects: "list[CacheSubscribedSubscribedType]" = betterproto.message_field(2)
    version: int = betterproto.fixed64_field(3)
    owner_soid: "IDOwner" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class CacheSubscribedSubscribedType(betterproto.Message):
    type_id: int = betterproto.int32_field(1)
    object_data: list[bytes] = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class CacheUnsubscribed(betterproto.Message):
    owner_soid: "IDOwner" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CacheSubscriptionCheck(betterproto.Message):
    version: int = betterproto.fixed64_field(2)
    owner_soid: "IDOwner" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CacheSubscriptionRefresh(betterproto.Message):
    owner_soid: "IDOwner" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CacheVersion(betterproto.Message):
    version: int = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class AccountDetails(betterproto.Message):
    valid: bool = betterproto.bool_field(1)
    account_name: str = betterproto.string_field(2)
    public_profile: bool = betterproto.bool_field(4)
    public_inventory: bool = betterproto.bool_field(5)
    vac_banned: bool = betterproto.bool_field(6)
    cyber_cafe: bool = betterproto.bool_field(7)
    school_account: bool = betterproto.bool_field(8)
    free_trial_account: bool = betterproto.bool_field(9)
    subscribed: bool = betterproto.bool_field(10)
    low_violence: bool = betterproto.bool_field(11)
    limited: bool = betterproto.bool_field(12)
    trusted: bool = betterproto.bool_field(13)
    package: int = betterproto.uint32_field(14)
    time_cached: int = betterproto.fixed32_field(15)
    account_locked: bool = betterproto.bool_field(16)
    community_banned: bool = betterproto.bool_field(17)
    trade_banned: bool = betterproto.bool_field(18)
    eligible_for_community_market: bool = betterproto.bool_field(19)


@dataclass(eq=False, repr=False)
class MultiplexMessage(betterproto.Message):
    msgtype: int = betterproto.uint32_field(1)
    payload: bytes = betterproto.bytes_field(2)
    steamids: list[int] = betterproto.fixed64_field(3)
    replytogc: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class MultiplexMessageResponse(betterproto.Message):
    msgtype: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class ToGcMsgMasterAck(betterproto.Message):
    dir_index: int = betterproto.uint32_field(1)
    gc_type: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class ToGcMsgMasterAckResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class ToGcMsgMasterStartupComplete(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ToGcMsgRouted(betterproto.Message):
    msg_type: int = betterproto.uint32_field(1)
    sender_id: int = betterproto.fixed64_field(2)
    net_message: bytes = betterproto.bytes_field(3)
    ip: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class ToGcMsgRoutedReply(betterproto.Message):
    msg_type: int = betterproto.uint32_field(1)
    net_message: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class UpdateSessionIp(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    ip: int = betterproto.fixed32_field(2)


@dataclass(eq=False, repr=False)
class RequestSessionIp(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class RequestSessionIpResponse(betterproto.Message):
    ip: int = betterproto.fixed32_field(1)


@dataclass(eq=False, repr=False)
class CacheHaveVersion(betterproto.Message):
    soid: "IDOwner" = betterproto.message_field(1)
    version: int = betterproto.fixed64_field(2)


class ClientHello(GCProtobufMessage, msg=EMsg.ClientHello):
    version: int = betterproto.uint32_field(1)
    socache_have_versions: "list[CacheHaveVersion]" = betterproto.message_field(2)
    client_session_need: int = betterproto.uint32_field(3)
    client_launcher: int = betterproto.uint32_field(4)
    partner_srcid: int = betterproto.uint32_field(5)
    partner_accountid: int = betterproto.uint32_field(6)
    partner_accountflags: int = betterproto.uint32_field(7)
    partner_accountbalance: int = betterproto.uint32_field(8)
    steam_launcher: int = betterproto.uint32_field(9)


@dataclass(eq=False, repr=False)
class ServerHello(betterproto.Message):
    version: int = betterproto.uint32_field(1)
    socache_have_versions: "list[CacheHaveVersion]" = betterproto.message_field(2)
    legacy_client_session_need: int = betterproto.uint32_field(3)
    client_launcher: int = betterproto.uint32_field(4)
    legacy_steamdatagram_routing: bytes = betterproto.bytes_field(6)
    required_internal_addr: int = betterproto.uint32_field(7)
    steamdatagram_login: bytes = betterproto.bytes_field(8)


class ClientWelcome(GCProtobufMessage, msg=EMsg.ClientWelcome):
    version: int = betterproto.uint32_field(1)
    game_data: bytes = betterproto.bytes_field(2)
    outofdate_subscribed_caches: "list[CacheSubscribed]" = betterproto.message_field(3)
    uptodate_subscribed_caches: "list[CacheSubscriptionCheck]" = betterproto.message_field(4)
    location: "ClientWelcomeLocation" = betterproto.message_field(5)
    game_data2: bytes = betterproto.bytes_field(6)
    rtime32_gc_welcome_timestamp: int = betterproto.uint32_field(7)
    currency: int = betterproto.uint32_field(8)
    balance: int = betterproto.uint32_field(9)
    balance_url: str = betterproto.string_field(10)
    txn_country_code: str = betterproto.string_field(11)


@dataclass(eq=False, repr=False)
class ClientWelcomeLocation(betterproto.Message):
    latitude: float = betterproto.float_field(1)
    longitude: float = betterproto.float_field(2)
    country: str = betterproto.string_field(3)


class ConnectionStatus(GCProtobufMessage, msg=EMsg.ClientConnectionStatus):
    status: "GcConnectionStatus" = betterproto.enum_field(1)
    client_session_need: int = betterproto.uint32_field(2)
    queue_position: int = betterproto.int32_field(3)
    queue_size: int = betterproto.int32_field(4)
    wait_seconds: int = betterproto.int32_field(5)
    estimated_wait_seconds_remaining: int = betterproto.int32_field(6)


@dataclass(eq=False, repr=False)
class PopulateItemDescriptionsRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    languages: "list[PopulateItemDescriptionsRequestItemDescriptionsLanguageBlock]" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class PopulateItemDescriptionsRequestSingleItemDescription(betterproto.Message):
    gameitemid: int = betterproto.uint32_field(1)
    item_description: str = betterproto.string_field(2)
    one_per_account: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class PopulateItemDescriptionsRequestItemDescriptionsLanguageBlock(betterproto.Message):
    language: str = betterproto.string_field(1)
    descriptions: "list[PopulateItemDescriptionsRequestSingleItemDescription]" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class GetContributorsRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    gameitemid: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class GetContributorsResponse(betterproto.Message):
    contributors: list[int] = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class SetItemPaymentRulesRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    gameitemid: int = betterproto.uint32_field(2)
    associated_workshop_files: "list[SetItemPaymentRulesRequestWorkshopItemPaymentRule]" = betterproto.message_field(3)
    partner_accounts: "list[SetItemPaymentRulesRequestPartnerItemPaymentRule]" = betterproto.message_field(4)
    validate_only: bool = betterproto.bool_field(5)
    make_workshop_files_subscribable: bool = betterproto.bool_field(6)
    associated_workshop_file_for_direct_payments: "SetItemPaymentRulesRequestWorkshopDirectPaymentRule" = (
        betterproto.message_field(7)
    )


@dataclass(eq=False, repr=False)
class SetItemPaymentRulesRequestWorkshopItemPaymentRule(betterproto.Message):
    workshop_file_id: int = betterproto.uint64_field(1)
    revenue_percentage: float = betterproto.float_field(2)
    rule_description: str = betterproto.string_field(3)
    rule_type: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class SetItemPaymentRulesRequestWorkshopDirectPaymentRule(betterproto.Message):
    workshop_file_id: int = betterproto.uint64_field(1)
    rule_description: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class SetItemPaymentRulesRequestPartnerItemPaymentRule(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    revenue_percentage: float = betterproto.float_field(2)
    rule_description: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class SetItemPaymentRulesResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CGameServersAggregationQueryRequest(betterproto.Message):
    filter: str = betterproto.string_field(1)
    group_fields: list[str] = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CGameServersAggregationQueryResponse(betterproto.Message):
    groups: "list[CGameServersAggregationQueryResponseGroup]" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CGameServersAggregationQueryResponseGroup(betterproto.Message):
    group_values: list[str] = betterproto.string_field(1)
    servers_empty: int = betterproto.uint32_field(2)
    servers_full: int = betterproto.uint32_field(3)
    servers_total: int = betterproto.uint32_field(4)
    players_humans: int = betterproto.uint32_field(5)
    players_bots: int = betterproto.uint32_field(6)
    player_capacity: int = betterproto.uint32_field(7)


@dataclass(eq=False, repr=False)
class AddSpecialPaymentRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    gameitemid: int = betterproto.uint32_field(2)
    date: str = betterproto.string_field(3)
    payment_us_usd: int = betterproto.uint64_field(4)
    payment_row_usd: int = betterproto.uint64_field(5)


@dataclass(eq=False, repr=False)
class AddSpecialPaymentResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class SetRichPresenceLocalizationRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    languages: "list[SetRichPresenceLocalizationRequestLanguageSection]" = betterproto.message_field(2)
    steamid: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class SetRichPresenceLocalizationRequestToken(betterproto.Message):
    token: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class SetRichPresenceLocalizationRequestLanguageSection(betterproto.Message):
    language: str = betterproto.string_field(1)
    tokens: "list[SetRichPresenceLocalizationRequestToken]" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class SetRichPresenceLocalizationResponse(betterproto.Message):
    pass



from __future__ import annotations

from typing import TYPE_CHECKING, Literal, TypeAlias, TypedDict

if TYPE_CHECKING:
    from typing_extensions import Required

    from ..enums import EventType


class Event(TypedDict, total=False):
    gid: Required[int]
    clan_steamid: Required[int]
    event_name: Required[str]
    event_type: Required[EventType]
    appid: Required[int]
    server_address: str
    server_password: str
    rtime32_start_time: Required[int]
    rtime32_end_time: int
    comment_count: int
    creator_steamid: Required[int]
    last_update_steamid: int
    event_notes: Required[str]
    jsondata: str
    announcement_body: Required[AnnouncementBody]
    published: bool
    hidden: bool
    rtime32_visibility_start: int
    rtime32_visibility_end: int
    broadcaster_accountid: int
    follower_count: int
    ignore_count: int
    forum_topic_id: Required[int]
    gidfeature: Required[int]
    gidfeature2: int
    rtime32_last_modified: int
    news_post_gid: int
    rtime_mod_reviewed: Required[int]
    featured_app_tagid: int
    referenced_appids: list[int]
    build_id: int
    build_branch: str


class AnnouncementBody(TypedDict):
    gid: int
    clanid: int
    posterid: int
    headline: str
    posttime: int
    updatetime: int
    body: str
    commentcount: int
    tags: list[str]
    language: int
    hidden: bool
    forum_topic_id: int
    event_gid: int
    voteupcount: int
    votedowncount: int


GetClanEvents: TypeAlias = dict[Literal["events"], list[Event]]
GetClanAnnouncement: TypeAlias = dict[Literal["event"], Event]



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

from __future__ import annotations

from typing import TYPE_CHECKING

from typing_extensions import Self, TypeVar

from .abc import BaseUser, Message
from .chat import ChatMessage, PartialMember
from .reaction import Emoticon, MessageReaction, Sticker
from .utils import DateTime

if TYPE_CHECKING:
    from .channel import ClanChannel, GroupChannel, UserChannel
    from .clan import Clan, ClanMember
    from .friend import Friend
    from .group import Group, GroupMember
    from .protobufs import chat, friend_messages
    from .user import ClientUser, User


__all__ = (
    "UserMessage",
    "GroupMessage",
    "ClanMessage",
)


UserMessageAuthorT = TypeVar("UserMessageAuthorT", bound=BaseUser, default="User | Friend | ClientUser", covariant=True)


class UserMessage(Message[UserMessageAuthorT, "UserChannel"]):
    """Represents a message from a user."""

    __slots__ = ()

    mentions: None

    def __init__(
        self, proto: friend_messages.IncomingMessageNotification, channel: UserChannel, author: UserMessageAuthorT
    ):
        super().__init__(channel, proto)
        self.created_at = DateTime.from_timestamp(proto.rtime32_server_timestamp)
        self.author = author

    async def _react(self, emoticon: Emoticon | Sticker, add: bool) -> None:
        await self._state.react_to_user_message(
            self.channel.participant.id64,
            int(self.created_at.timestamp()),
            self.ordinal,
            str(emoticon) if isinstance(emoticon, Emoticon) else emoticon.name,
            reaction_type=emoticon._TYPE,
            is_add=add,
        )
        reaction = MessageReaction(
            self._state,
            self,
            emoticon if isinstance(emoticon, Emoticon) else None,  # type: ignore
            emoticon if isinstance(emoticon, Sticker) else None,  # type: ignore
            self._state.user,
            DateTime.now(),
            self.ordinal,
        )
        if add:
            self.reactions.append(reaction)
        else:
            self.reactions.remove(reaction)
        self._state.dispatch(f"reaction_{'add' if add else 'remove'}", reaction)

    @classmethod
    def _from_history(cls, channel: UserChannel, proto: friend_messages.GetRecentMessagesResponseFriendMessage) -> Self:
        self = cls.__new__(cls)  # skip __init__
        super().__init__(self, channel, proto)
        self.created_at = DateTime.from_timestamp(proto.timestamp)
        return self

    async def ack(self) -> None:
        await self._state.ack_user_message(self.channel.participant.id64, int(self.created_at.timestamp()))


GroupMessageAuthorT = TypeVar(
    "GroupMessageAuthorT", bound="PartialMember", default="PartialMember | GroupMember", covariant=True
)


class GroupMessage(ChatMessage[GroupMessageAuthorT, "GroupMember", "GroupChannel"]):
    """Represents a message in a group."""

    __slots__ = ()

    group: Group
    clan: None

    def __init__(self, proto: chat.IncomingChatMessageNotification, channel: GroupChannel, author: GroupMessageAuthorT):
        super().__init__(proto, channel, author)


ClanMessageAuthorT = TypeVar(
    "ClanMessageAuthorT", bound="PartialMember", default="PartialMember | ClanMember", covariant=True
)


class ClanMessage(ChatMessage[ClanMessageAuthorT, "ClanMember", "ClanChannel"]):
    """Represents a message in a clan."""

    __slots__ = ()

    clan: Clan
    group: None

    def __init__(self, proto: chat.IncomingChatMessageNotification, channel: ClanChannel, author: ClanMessageAuthorT):
        super().__init__(proto, channel, author)



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

from __future__ import annotations

import abc
import sys
import traceback
from copy import copy
from typing import TYPE_CHECKING, cast

from typing_extensions import Unpack, final

from .commands import Command, CommandKwargs, Group
from .context import Context  # noqa: TC001

if TYPE_CHECKING:
    from collections.abc import Mapping, Sequence

    from steam.ext import commands


__all__ = (
    "HelpCommand",
    "DefaultHelpCommand",
)


class HelpCommand(Command[None, ..., None]):
    """The base implementation of the help command."""

    context: Context
    """The context for the command's invocation."""

    def __init__(self, **kwargs: Unpack[CommandKwargs]):
        default = cast(CommandKwargs, {"name": "help", "help": "Shows this message."} | kwargs)
        super().__init__(self.command_callback, **default)

    async def invoke(self, ctx: Context) -> None:
        self = copy(self)
        ctx.command = self
        return await Command.invoke(self, ctx)  # type: ignore

    @final
    async def command_callback(self, ctx: Context, *, content: str | None = None) -> None:
        """The actual implementation of the help command.

        This method should not directly subclassed instead you should change the behaviour through the methods that
        actually get dispatched:

            - :meth:`send_cog_help`
            - :meth:`send_command_help`
            - :meth:`send_group_help`
            - :meth:`command_not_found`
        """
        self.context = ctx
        bot = ctx.bot
        if content is None:
            mapping = self.get_bot_mapping()
            return await self.send_help(mapping)
        # check if it's a cog
        cog = bot.get_cog(content)
        if cog is not None:
            return await self.send_cog_help(cog)
        command = bot.get_command(content)
        if command is not None:
            return await (
                self.send_group_help(command) if isinstance(command, Group) else self.send_command_help(command)
            )

        await self.command_not_found(content)

    def get_bot_mapping(self) -> Mapping[str | None, Sequence[commands.Command]]:
        """
        Generate a mapping of the bot's commands. It's not normally necessary to subclass this. This is passed to
        :meth:`send_help`.
        """
        bot = self.context.bot
        return {cog.qualified_name: list(cog.commands) for cog in bot.cogs.values() if cog.commands} | {
            None: [c for c in bot.commands if c.cog is None]
        }

    @abc.abstractmethod
    async def send_help(self, mapping: Mapping[str | None, Sequence[commands.Command]]) -> None:
        """Send the basic help message for the bot's command.

        Parameters
        ----------
        mapping
            The mapping from :meth:`get_bot_mapping`.
        """

    @abc.abstractmethod
    async def send_cog_help(self, cog: commands.Cog) -> None:
        """The method called with a cog is passed as an argument.

        Note
        ----
        Cog names are case-sensitive.

        Parameters
        ----------
        cog
            The cog that was passed as an argument.
        """

    @abc.abstractmethod
    async def send_command_help(self, command: commands.Command) -> None:
        """The method called when a normal command is passed as an argument.

        Parameters
        ----------
        command
            The command that was passed as an argument.
        """

    @abc.abstractmethod
    async def send_group_help(self, command: commands.Group) -> None:
        """The method called when a group command is passed as an argument.

        Parameters
        ----------
        command
            The command that was passed as an argument.
        """

    @abc.abstractmethod
    async def command_not_found(self, command: str) -> None:
        """The default implementation for when a command isn't found.

        This by default sends "The command {command} was not found."

        Parameters
        ----------
        command
            The command that was not found.
        """

    async def on_error(self, ctx: commands.Context, error: Exception) -> None:
        """The default error handler for the help command. This performs the functionality as
        :meth:`steam.ext.commands.Bot.on_command_error`.

        Parameters
        ----------
        ctx
            The context for the invocation.
        error
            The error that was raised.
        """
        print(f"Ignoring exception in command {ctx.command}:", file=sys.stderr)
        traceback.print_exception(type(error), error, error.__traceback__, file=sys.stderr)


class DefaultHelpCommand(HelpCommand):
    """The default implementation of the help command."""

    def __repr__(self) -> str:
        return "<default_help_command>"

    def _get_doc(self, command: Command) -> str:
        if command.help:
            try:
                return command.help.splitlines()[0]
            except IndexError:
                pass
        return ""

    async def send_help(self, mapping: Mapping[str | None, Sequence[commands.Command]]) -> None:
        message = ["/pre"]
        for cog_name, commands in mapping.items():
            (
                message.append(f"\n{cog_name}'s commands")
                if cog_name is not None
                else message.append("\nUn-categorized commands")
            )
            message += (
                f'{command.name}{f": {self._get_doc(command)}" if command.help else ""}' for command in commands
            )

        await self.context.send("\n".join(message))

    async def send_cog_help(self, cog: commands.Cog) -> None:
        message = [f"/pre {cog.qualified_name}'s commands"]
        for name in sorted(c.name for c in cog.commands):
            command = cog.__commands__[name]
            message.append(f'{name}{f": {self._get_doc(command)}" if command.help else ""}')
        await self.context.send("\n".join(message))

    async def send_command_help(self, command: commands.Command) -> None:
        await self.context.send(f"/pre Help with {command.name}:\n\n{command.help}")

    async def send_group_help(self, command: commands.Group) -> None:
        msg = [f"/pre Help with {command.name}:\n\n{command.help}"]
        if sub_commands := "\n".join(c.name for c in command.children):
            msg.append(f"\nAnd its sub commands:\n{sub_commands}")
        await self.context.send("\n".join(msg))

    async def command_not_found(self, command: str) -> None:
        """The default implementation for when a command isn't found.

        This by default sends "The command {command} was not found."

        Parameters
        ----------
        command
            The command that was not found.
        """
        await self.context.send(f"The command {command!r} was not found.")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cstrike15_gcmessages.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import TYPE_CHECKING, Union

import betterproto

from ....protobufs.msg import GCProtobufMessage
from ..enums import EMsg

if TYPE_CHECKING:
    from .engine import GotvSyncPacket


class ECsgoSteamUserStat(betterproto.Enum):
    XPEarnedGames = 1
    MatchWinsCompetitive = 2
    SurvivedDangerZone = 3


class EClientReportingVersion(betterproto.Enum):
    OldVersion = 0
    BetaVersion = 1
    SupportsTrustedMode = 2


@dataclass(eq=False, repr=False)
class GameServerPing(betterproto.Message):
    ping: int = betterproto.int32_field(2)
    ip: int = betterproto.uint32_field(3)
    instances: int = betterproto.uint32_field(5)


@dataclass(eq=False, repr=False)
class DataCenterPing(betterproto.Message):
    data_center_id: int = betterproto.fixed32_field(1)
    ping: int = betterproto.sint32_field(2)


@dataclass(eq=False, repr=False)
class DetailedSearchStatistic(betterproto.Message):
    game_type: int = betterproto.uint32_field(1)
    search_time_avg: int = betterproto.uint32_field(2)
    players_searching: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class TournamentPlayer(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    player_nick: str = betterproto.string_field(2)
    player_name: str = betterproto.string_field(3)
    player_dob: int = betterproto.uint32_field(4)
    player_flag: str = betterproto.string_field(5)
    player_location: str = betterproto.string_field(6)
    player_desc: str = betterproto.string_field(7)


@dataclass(eq=False, repr=False)
class TournamentTeam(betterproto.Message):
    team_id: int = betterproto.int32_field(1)
    team_tag: str = betterproto.string_field(2)
    team_flag: str = betterproto.string_field(3)
    team_name: str = betterproto.string_field(4)
    players: "list[TournamentPlayer]" = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class TournamentEvent(betterproto.Message):
    event_id: int = betterproto.int32_field(1)
    event_tag: str = betterproto.string_field(2)
    event_name: str = betterproto.string_field(3)
    event_time_start: int = betterproto.uint32_field(4)
    event_time_end: int = betterproto.uint32_field(5)
    event_public: int = betterproto.int32_field(6)
    event_stage_id: int = betterproto.int32_field(7)
    event_stage_name: str = betterproto.string_field(8)
    active_section_id: int = betterproto.uint32_field(9)


@dataclass(eq=False, repr=False)
class GlobalStatistics(betterproto.Message):
    players_online: int = betterproto.uint32_field(1)
    servers_online: int = betterproto.uint32_field(2)
    players_searching: int = betterproto.uint32_field(3)
    servers_available: int = betterproto.uint32_field(4)
    ongoing_matches: int = betterproto.uint32_field(5)
    search_time_avg: int = betterproto.uint32_field(6)
    search_statistics: "list[DetailedSearchStatistic]" = betterproto.message_field(7)
    main_post_url: str = betterproto.string_field(8)
    required_appid_version: int = betterproto.uint32_field(9)
    pricesheet_version: int = betterproto.uint32_field(10)
    twitch_streams_version: int = betterproto.uint32_field(11)
    active_tournament_eventid: int = betterproto.uint32_field(12)
    active_survey_id: int = betterproto.uint32_field(13)
    rtime32_cur: int = betterproto.uint32_field(14)
    rtime32_event_start: int = betterproto.uint32_field(15)


@dataclass(eq=False, repr=False)
class OperationalStatisticDescription(betterproto.Message):
    name: str = betterproto.string_field(1)
    idkey: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class OperationalStatisticElement(betterproto.Message):
    idkey: int = betterproto.uint32_field(1)
    values: list[int] = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class OperationalStatisticsPacket(betterproto.Message):
    packetid: int = betterproto.int32_field(1)
    mstimestamp: int = betterproto.int32_field(2)
    values: "list[OperationalStatisticElement]" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class PlayerRankingInfo(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    rank_id: int = betterproto.uint32_field(2)
    wins: int = betterproto.uint32_field(3)
    rank_change: float = betterproto.float_field(4)
    rank_type_id: int = betterproto.uint32_field(6)
    tv_control: int = betterproto.uint32_field(7)


@dataclass(eq=False, repr=False)
class PlayerCommendationInfo(betterproto.Message):
    cmd_friendly: int = betterproto.uint32_field(1)
    cmd_teaching: int = betterproto.uint32_field(2)
    cmd_leader: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class PlayerMedalsInfo(betterproto.Message):
    display_items_defidx: list[int] = betterproto.uint32_field(7)
    featured_display_item_defidx: int = betterproto.uint32_field(8)


@dataclass(eq=False, repr=False)
class AccountActivity(betterproto.Message):
    activity: int = betterproto.uint32_field(1)
    mode: int = betterproto.uint32_field(2)
    map: int = betterproto.uint32_field(3)
    matchid: int = betterproto.uint64_field(4)


@dataclass(eq=False, repr=False)
class TournamentMatchSetup(betterproto.Message):
    event_id: int = betterproto.int32_field(1)
    team_id_ct: int = betterproto.int32_field(2)
    team_id_t: int = betterproto.int32_field(3)
    event_stage_id: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class ServerHltvInfo(betterproto.Message):
    tv_udp_port: int = betterproto.uint32_field(1)
    tv_watch_key: int = betterproto.uint64_field(2)
    tv_slots: int = betterproto.uint32_field(3)
    tv_clients: int = betterproto.uint32_field(4)
    tv_proxies: int = betterproto.uint32_field(5)
    tv_time: int = betterproto.uint32_field(6)
    game_type: int = betterproto.uint32_field(8)
    game_mapgroup: str = betterproto.string_field(9)
    game_map: str = betterproto.string_field(10)
    tv_master_steamid: int = betterproto.uint64_field(11)
    tv_local_slots: int = betterproto.uint32_field(12)
    tv_local_clients: int = betterproto.uint32_field(13)
    tv_local_proxies: int = betterproto.uint32_field(14)
    tv_relay_slots: int = betterproto.uint32_field(15)
    tv_relay_clients: int = betterproto.uint32_field(16)
    tv_relay_proxies: int = betterproto.uint32_field(17)
    tv_relay_address: int = betterproto.uint32_field(18)
    tv_relay_port: int = betterproto.uint32_field(19)
    tv_relay_steamid: int = betterproto.uint64_field(20)
    flags: int = betterproto.uint32_field(21)


@dataclass(eq=False, repr=False)
class IpAddressMask(betterproto.Message):
    a: int = betterproto.uint32_field(1)
    b: int = betterproto.uint32_field(2)
    c: int = betterproto.uint32_field(3)
    d: int = betterproto.uint32_field(4)
    bits: int = betterproto.uint32_field(5)
    token: int = betterproto.uint32_field(6)


@dataclass(eq=False, repr=False)
class CsgoSteamUserStatChange(betterproto.Message):
    ecsgosteamuserstat: "ECsgoSteamUserStat" = betterproto.enum_field(1)
    delta: int = betterproto.int32_field(2)
    absolute: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class XpProgressData(betterproto.Message):
    xp_points: int = betterproto.uint32_field(1)
    xp_category: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class MatchEndItemUpdates(betterproto.Message):
    item_id: int = betterproto.uint64_field(1)
    item_attr_defidx: int = betterproto.uint32_field(2)
    item_attr_delta_value: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class ScoreLeaderboardData(betterproto.Message):
    quest_id: int = betterproto.uint64_field(1)
    score: int = betterproto.uint32_field(2)
    accountentries: "list[ScoreLeaderboardDataAccountEntries]" = betterproto.message_field(3)
    matchentries: "list[ScoreLeaderboardDataEntry]" = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class ScoreLeaderboardDataEntry(betterproto.Message):
    tag: int = betterproto.uint32_field(1)
    val: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class ScoreLeaderboardDataAccountEntries(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    entries: "list[ScoreLeaderboardDataEntry]" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class PlayerQuestData(betterproto.Message):
    quester_account_id: int = betterproto.uint32_field(1)
    quest_item_data: "list[PlayerQuestDataQuestItemData]" = betterproto.message_field(2)
    xp_progress_data: "list[XpProgressData]" = betterproto.message_field(3)
    time_played: int = betterproto.uint32_field(4)
    mm_game_mode: int = betterproto.uint32_field(5)
    item_updates: "list[MatchEndItemUpdates]" = betterproto.message_field(6)
    operation_points_eligible: bool = betterproto.bool_field(7)
    userstatchanges: "list[CsgoSteamUserStatChange]" = betterproto.message_field(8)


@dataclass(eq=False, repr=False)
class PlayerQuestDataQuestItemData(betterproto.Message):
    quest_id: int = betterproto.uint64_field(1)
    quest_normal_points_earned: int = betterproto.int32_field(2)
    quest_bonus_points_earned: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class DeepPlayerStatsEntry(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    match_id: int = betterproto.uint64_field(2)
    mm_game_mode: int = betterproto.uint32_field(3)
    mapid: int = betterproto.uint32_field(4)
    b_starting_ct: bool = betterproto.bool_field(5)
    match_outcome: int = betterproto.uint32_field(6)
    rounds_won: int = betterproto.uint32_field(7)
    rounds_lost: int = betterproto.uint32_field(8)
    stat_score: int = betterproto.uint32_field(9)
    stat_deaths: int = betterproto.uint32_field(12)
    stat_mvps: int = betterproto.uint32_field(13)
    enemy_kills: int = betterproto.uint32_field(14)
    enemy_headshots: int = betterproto.uint32_field(15)
    enemy_2_ks: int = betterproto.uint32_field(16)
    enemy_3_ks: int = betterproto.uint32_field(17)
    enemy_4_ks: int = betterproto.uint32_field(18)
    total_damage: int = betterproto.uint32_field(19)
    engagements_entry_count: int = betterproto.uint32_field(23)
    engagements_entry_wins: int = betterproto.uint32_field(24)
    engagements_1_v1_count: int = betterproto.uint32_field(25)
    engagements_1_v1_wins: int = betterproto.uint32_field(26)
    engagements_1_v2_count: int = betterproto.uint32_field(27)
    engagements_1_v2_wins: int = betterproto.uint32_field(28)
    utility_count: int = betterproto.uint32_field(29)
    utility_success: int = betterproto.uint32_field(30)
    flash_count: int = betterproto.uint32_field(32)
    flash_success: int = betterproto.uint32_field(33)
    mates: list[int] = betterproto.uint32_field(34)


@dataclass(eq=False, repr=False)
class DeepPlayerMatchEvent(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    match_id: int = betterproto.uint64_field(2)
    event_id: int = betterproto.uint32_field(3)
    event_type: int = betterproto.uint32_field(4)
    b_playing_ct: bool = betterproto.bool_field(5)
    user_pos_x: int = betterproto.int32_field(6)
    user_pos_y: int = betterproto.int32_field(7)
    user_pos_z: int = betterproto.int32_field(12)
    user_defidx: int = betterproto.uint32_field(8)
    other_pos_x: int = betterproto.int32_field(9)
    other_pos_y: int = betterproto.int32_field(10)
    other_pos_z: int = betterproto.int32_field(13)
    other_defidx: int = betterproto.uint32_field(11)
    event_data: int = betterproto.int32_field(14)


@dataclass(eq=False, repr=False)
class GcServerQuestUpdateData(betterproto.Message):
    player_quest_data: "list[PlayerQuestData]" = betterproto.message_field(1)
    binary_data: bytes = betterproto.bytes_field(2)
    mm_game_mode: int = betterproto.uint32_field(3)
    missionlbsdata: "ScoreLeaderboardData" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class MatchmakingGcOperationalStats(betterproto.Message):
    packetid: int = betterproto.int32_field(1)
    namekeys: "list[OperationalStatisticDescription]" = betterproto.message_field(2)
    packets: "list[OperationalStatisticsPacket]" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class MatchmakingServerConfirm(betterproto.Message):
    token: int = betterproto.uint32_field(1)
    stamp: int = betterproto.uint32_field(2)
    exchange: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class ServerReservationUpdate(betterproto.Message):
    viewers_external_total: int = betterproto.uint32_field(1)
    viewers_external_steam: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class MatchmakingStart(betterproto.Message):
    account_ids: list[int] = betterproto.uint32_field(1)
    game_type: int = betterproto.uint32_field(2)
    ticket_data: str = betterproto.string_field(3)
    client_version: int = betterproto.uint32_field(4)
    tournament_match: "TournamentMatchSetup" = betterproto.message_field(5)
    prime_only: bool = betterproto.bool_field(6)


@dataclass(eq=False, repr=False)
class MatchmakingStop(betterproto.Message):
    abandon: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class MatchmakingClient2ServerPing(betterproto.Message):
    gameserverpings: "list[GameServerPing]" = betterproto.message_field(1)
    offset_index: int = betterproto.int32_field(2)
    final_batch: int = betterproto.int32_field(3)
    data_center_pings: "list[DataCenterPing]" = betterproto.message_field(4)
    max_ping: int = betterproto.uint32_field(5)
    test_token: int = betterproto.fixed32_field(6)


@dataclass(eq=False, repr=False)
class MatchmakingClientUpdate(betterproto.Message):
    matchmaking: int = betterproto.int32_field(1)
    waiting_account_id_sessions: list[int] = betterproto.uint32_field(2)
    error: str = betterproto.string_field(3)
    ongoingmatch_account_id_sessions: list[int] = betterproto.uint32_field(6)
    global_stats: "GlobalStatistics" = betterproto.message_field(7)
    failping_account_id_sessions: list[int] = betterproto.uint32_field(8)
    penalty_account_id_sessions: list[int] = betterproto.uint32_field(9)
    failready_account_id_sessions: list[int] = betterproto.uint32_field(10)
    vacbanned_account_id_sessions: list[int] = betterproto.uint32_field(11)
    server_ipaddress_mask: "IpAddressMask" = betterproto.message_field(12)
    notes: "list[MatchmakingClientUpdateNote]" = betterproto.message_field(13)
    penalty_account_id_sessions_green: list[int] = betterproto.uint32_field(14)
    insufficientlevel_sessions: list[int] = betterproto.uint32_field(15)
    vsncheck_account_id_sessions: list[int] = betterproto.uint32_field(16)
    launcher_mismatch_sessions: list[int] = betterproto.uint32_field(17)
    insecure_account_id_sessions: list[int] = betterproto.uint32_field(18)


@dataclass(eq=False, repr=False)
class MatchmakingClientUpdateNote(betterproto.Message):
    type: int = betterproto.int32_field(1)
    region_id: int = betterproto.int32_field(2)
    region_r: float = betterproto.float_field(3)
    distance: float = betterproto.float_field(4)


@dataclass(eq=False, repr=False)
class TournamentMatchDraft(betterproto.Message):
    event_id: int = betterproto.int32_field(1)
    event_stage_id: int = betterproto.int32_field(2)
    team_id_0: int = betterproto.int32_field(3)
    team_id_1: int = betterproto.int32_field(4)
    maps_count: int = betterproto.int32_field(5)
    maps_current: int = betterproto.int32_field(6)
    team_id_start: int = betterproto.int32_field(7)
    team_id_veto1: int = betterproto.int32_field(8)
    team_id_pickn: int = betterproto.int32_field(9)
    drafts: "list[TournamentMatchDraftEntry]" = betterproto.message_field(10)


@dataclass(eq=False, repr=False)
class TournamentMatchDraftEntry(betterproto.Message):
    mapid: int = betterproto.int32_field(1)
    team_id_ct: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class CPreMatchInfoData(betterproto.Message):
    predictions_pct: int = betterproto.int32_field(1)
    draft: "TournamentMatchDraft" = betterproto.message_field(4)
    stats: "list[CPreMatchInfoDataTeamStats]" = betterproto.message_field(5)
    wins: list[int] = betterproto.int32_field(6)


@dataclass(eq=False, repr=False)
class CPreMatchInfoDataTeamStats(betterproto.Message):
    match_info_idxtxt: int = betterproto.int32_field(1)
    match_info_txt: str = betterproto.string_field(2)
    match_info_teams: list[str] = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class MatchmakingServerReserve(betterproto.Message):
    account_ids: list[int] = betterproto.uint32_field(1)
    game_type: int = betterproto.uint32_field(2)
    match_id: int = betterproto.uint64_field(3)
    server_version: int = betterproto.uint32_field(4)
    flags: int = betterproto.uint32_field(18)
    rankings: "list[PlayerRankingInfo]" = betterproto.message_field(5)
    encryption_key: int = betterproto.uint64_field(6)
    encryption_key_pub: int = betterproto.uint64_field(7)
    party_ids: list[int] = betterproto.uint32_field(8)
    whitelist: "list[IpAddressMask]" = betterproto.message_field(9)
    tv_master_steamid: int = betterproto.uint64_field(10)
    tournament_event: "TournamentEvent" = betterproto.message_field(11)
    tournament_teams: "list[TournamentTeam]" = betterproto.message_field(12)
    tournament_casters_account_ids: list[int] = betterproto.uint32_field(13)
    tv_relay_steamid: int = betterproto.uint64_field(14)
    pre_match_data: "CPreMatchInfoData" = betterproto.message_field(15)
    rtime32_event_start: int = betterproto.uint32_field(16)
    tv_control: int = betterproto.uint32_field(17)


@dataclass(eq=False, repr=False)
class MatchmakingServerReservationResponse(betterproto.Message):
    reservationid: int = betterproto.uint64_field(1)
    reservation: "MatchmakingServerReserve" = betterproto.message_field(2)
    map: str = betterproto.string_field(3)
    gc_reservation_sent: int = betterproto.uint64_field(4)
    server_version: int = betterproto.uint32_field(5)
    tv_info: "ServerHltvInfo" = betterproto.message_field(6)
    reward_player_accounts: list[int] = betterproto.uint32_field(7)
    idle_player_accounts: list[int] = betterproto.uint32_field(8)
    reward_item_attr_def_idx: int = betterproto.uint32_field(9)
    reward_item_attr_value: int = betterproto.uint32_field(10)
    reward_item_attr_reward_idx: int = betterproto.uint32_field(11)
    reward_drop_list: int = betterproto.uint32_field(12)
    tournament_tag: str = betterproto.string_field(13)
    legacy_steamdatagram_port: int = betterproto.uint32_field(14)
    steamdatagram_routing: int = betterproto.uint32_field(17)
    test_token: int = betterproto.fixed32_field(15)
    flags: int = betterproto.uint32_field(16)


@dataclass(eq=False, repr=False)
class MatchmakingClientReserve(betterproto.Message):
    serverid: int = betterproto.uint64_field(1)
    direct_udp_ip: int = betterproto.uint32_field(2)
    direct_udp_port: int = betterproto.uint32_field(3)
    reservationid: int = betterproto.uint64_field(4)
    reservation: "MatchmakingServerReserve" = betterproto.message_field(5)
    map: str = betterproto.string_field(6)
    server_address: str = betterproto.string_field(7)


@dataclass(eq=False, repr=False)
class MatchmakingServerRoundStats(betterproto.Message):
    reservationid: int = betterproto.uint64_field(1)
    reservation: "MatchmakingServerReserve" = betterproto.message_field(2)
    map: str = betterproto.string_field(3)
    round: int = betterproto.int32_field(4)
    kills: list[int] = betterproto.int32_field(5)
    assists: list[int] = betterproto.int32_field(6)
    deaths: list[int] = betterproto.int32_field(7)
    scores: list[int] = betterproto.int32_field(8)
    pings: list[int] = betterproto.int32_field(9)
    round_result: int = betterproto.int32_field(10)
    match_result: int = betterproto.int32_field(11)
    team_scores: list[int] = betterproto.int32_field(12)
    confirm: "MatchmakingServerConfirm" = betterproto.message_field(13)
    reservation_stage: int = betterproto.int32_field(14)
    match_duration: int = betterproto.int32_field(15)
    enemy_kills: list[int] = betterproto.int32_field(16)
    enemy_headshots: list[int] = betterproto.int32_field(17)
    enemy_3_ks: list[int] = betterproto.int32_field(18)
    enemy_4_ks: list[int] = betterproto.int32_field(19)
    enemy_5_ks: list[int] = betterproto.int32_field(20)
    mvps: list[int] = betterproto.int32_field(21)
    spectators_count: int = betterproto.uint32_field(22)
    spectators_count_tv: int = betterproto.uint32_field(23)
    spectators_count_lnk: int = betterproto.uint32_field(24)
    enemy_kills_agg: list[int] = betterproto.int32_field(25)
    drop_info: "MatchmakingServerRoundStatsDropInfo" = betterproto.message_field(26)
    b_switched_teams: bool = betterproto.bool_field(27)
    enemy_2_ks: list[int] = betterproto.int32_field(28)
    player_spawned: list[int] = betterproto.int32_field(29)
    team_spawn_count: list[int] = betterproto.int32_field(30)


@dataclass(eq=False, repr=False)
class MatchmakingServerRoundStatsDropInfo(betterproto.Message):
    account_mvp: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class MatchmakingClient2GcHello(betterproto.Message):
    pass


class MatchmakingClientHello(GCProtobufMessage, msg=EMsg.MatchmakingGC2ClientHello):
    account_id: int = betterproto.uint32_field(1)
    ongoingmatch: "MatchmakingClientReserve" = betterproto.message_field(2)
    global_stats: "GlobalStatistics" = betterproto.message_field(3)
    penalty_seconds: int = betterproto.uint32_field(4)
    penalty_reason: int = betterproto.uint32_field(5)
    vac_banned: int = betterproto.int32_field(6)
    ranking: "PlayerRankingInfo" = betterproto.message_field(7)
    commendation: "PlayerCommendationInfo" = betterproto.message_field(8)
    medals: "PlayerMedalsInfo" = betterproto.message_field(9)
    my_current_event: "TournamentEvent" = betterproto.message_field(10)
    my_current_event_teams: "list[TournamentTeam]" = betterproto.message_field(11)
    my_current_team: "TournamentTeam" = betterproto.message_field(12)
    my_current_event_stages: "list[TournamentEvent]" = betterproto.message_field(13)
    survey_vote: int = betterproto.uint32_field(14)
    activity: "AccountActivity" = betterproto.message_field(15)
    player_level: int = betterproto.int32_field(17)
    player_cur_xp: int = betterproto.int32_field(18)
    player_xp_bonus_flags: int = betterproto.int32_field(19)
    rankings: "list[PlayerRankingInfo]" = betterproto.message_field(20)


@dataclass(eq=False, repr=False)
class AccountPrivacySettings(betterproto.Message):
    settings: "list[AccountPrivacySettingsSetting]" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class AccountPrivacySettingsSetting(betterproto.Message):
    setting_type: int = betterproto.uint32_field(1)
    setting_value: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class MatchmakingClientAbandon(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    abandoned_match: "MatchmakingClientReserve" = betterproto.message_field(2)
    penalty_seconds: int = betterproto.uint32_field(3)
    penalty_reason: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class ClientGcRankUpdate(betterproto.Message):
    rankings: "list[PlayerRankingInfo]" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class MatchmakingOperator2GcBlogUpdate(betterproto.Message):
    main_post_url: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ServerNotificationForUserPenalty(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    reason: int = betterproto.uint32_field(2)
    seconds: int = betterproto.uint32_field(3)
    communication_cooldown: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class ClientReportPlayer(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    rpt_aimbot: int = betterproto.uint32_field(2)
    rpt_wallhack: int = betterproto.uint32_field(3)
    rpt_speedhack: int = betterproto.uint32_field(4)
    rpt_teamharm: int = betterproto.uint32_field(5)
    rpt_textabuse: int = betterproto.uint32_field(6)
    rpt_voiceabuse: int = betterproto.uint32_field(7)
    match_id: int = betterproto.uint64_field(8)
    report_from_demo: bool = betterproto.bool_field(9)


@dataclass(eq=False, repr=False)
class ClientCommendPlayer(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    match_id: int = betterproto.uint64_field(8)
    commendation: "PlayerCommendationInfo" = betterproto.message_field(9)
    tokens: int = betterproto.uint32_field(10)


@dataclass(eq=False, repr=False)
class ClientReportServer(betterproto.Message):
    rpt_poorperf: int = betterproto.uint32_field(1)
    rpt_abusivemodels: int = betterproto.uint32_field(2)
    rpt_badmotd: int = betterproto.uint32_field(3)
    rpt_listingabuse: int = betterproto.uint32_field(4)
    rpt_inventoryabuse: int = betterproto.uint32_field(5)
    match_id: int = betterproto.uint64_field(8)


@dataclass(eq=False, repr=False)
class ClientReportResponse(betterproto.Message):
    confirmation_id: int = betterproto.uint64_field(1)
    account_id: int = betterproto.uint32_field(2)
    server_ip: int = betterproto.uint32_field(3)
    response_type: int = betterproto.uint32_field(4)
    response_result: int = betterproto.uint32_field(5)
    tokens: int = betterproto.uint32_field(6)


class ClientRequestWatchInfoFriends(GCProtobufMessage, msg=EMsg.ClientRequestWatchInfoFriends2):
    request_id: int = betterproto.uint32_field(1)
    account_ids: list[int] = betterproto.uint32_field(2)
    serverid: int = betterproto.uint64_field(3)
    matchid: int = betterproto.uint64_field(4)
    client_launcher: int = betterproto.uint32_field(5)
    data_center_pings: "list[DataCenterPing]" = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class WatchableMatchInfo(betterproto.Message):
    server_ip: int = betterproto.uint32_field(1)
    tv_port: int = betterproto.uint32_field(2)
    tv_spectators: int = betterproto.uint32_field(3)
    tv_time: int = betterproto.uint32_field(4)
    tv_watch_password: bytes = betterproto.bytes_field(5)
    cl_decryptdata_key: int = betterproto.uint64_field(6)
    cl_decryptdata_key_pub: int = betterproto.uint64_field(7)
    game_type: int = betterproto.uint32_field(8)
    game_mapgroup: str = betterproto.string_field(9)
    game_map: str = betterproto.string_field(10)
    server_id: int = betterproto.uint64_field(11)
    match_id: int = betterproto.uint64_field(12)
    reservation_id: int = betterproto.uint64_field(13)


@dataclass(eq=False, repr=False)
class ClientRequestJoinFriendData(betterproto.Message):
    version: int = betterproto.uint32_field(1)
    account_id: int = betterproto.uint32_field(2)
    join_token: int = betterproto.uint32_field(3)
    join_ipp: int = betterproto.uint32_field(4)
    res: "MatchmakingClientReserve" = betterproto.message_field(5)
    errormsg: str = betterproto.string_field(6)


@dataclass(eq=False, repr=False)
class ClientRequestJoinServerData(betterproto.Message):
    version: int = betterproto.uint32_field(1)
    account_id: int = betterproto.uint32_field(2)
    serverid: int = betterproto.uint64_field(3)
    server_ip: int = betterproto.uint32_field(4)
    server_port: int = betterproto.uint32_field(5)
    res: "MatchmakingClientReserve" = betterproto.message_field(6)
    errormsg: str = betterproto.string_field(7)


@dataclass(eq=False, repr=False)
class ClientRequestNewMission(betterproto.Message):
    mission_id: int = betterproto.uint32_field(2)
    campaign_id: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class ClientRedeemMissionReward(betterproto.Message):
    campaign_id: int = betterproto.uint32_field(1)
    redeem_id: int = betterproto.uint32_field(2)
    redeemable_balance: int = betterproto.uint32_field(3)
    expected_cost: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class ServerNotifyXpRewarded(betterproto.Message):
    xp_progress_data: "list[XpProgressData]" = betterproto.message_field(1)
    account_id: int = betterproto.uint32_field(2)
    current_xp: int = betterproto.uint32_field(3)
    current_level: int = betterproto.uint32_field(4)
    upgraded_defidx: int = betterproto.uint32_field(5)
    operation_points_awarded: int = betterproto.uint32_field(6)


@dataclass(eq=False, repr=False)
class ClientDeepStats(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    range: "ClientDeepStatsDeepStatsRange" = betterproto.message_field(2)
    matches: "list[ClientDeepStatsDeepStatsMatch]" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class ClientDeepStatsDeepStatsRange(betterproto.Message):
    begin: int = betterproto.uint32_field(1)
    end: int = betterproto.uint32_field(2)
    frozen: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class ClientDeepStatsDeepStatsMatch(betterproto.Message):
    player: "DeepPlayerStatsEntry" = betterproto.message_field(1)
    events: "list[DeepPlayerMatchEvent]" = betterproto.message_field(2)


class WatchInfoUsers(GCProtobufMessage, msg=EMsg.WatchInfoUsers):
    request_id: int = betterproto.uint32_field(1)
    account_ids: list[int] = betterproto.uint32_field(2)
    watchable_match_infos: "list[WatchableMatchInfo]" = betterproto.message_field(3)
    extended_timeout: int = betterproto.uint32_field(5)


class ClientRequestPlayersProfile(GCProtobufMessage, msg=EMsg.ClientRequestPlayersProfile):
    request_id_deprecated: int = betterproto.uint32_field(1)
    account_ids_deprecated: list[int] = betterproto.uint32_field(2)
    account_id: int = betterproto.uint32_field(3)
    request_level: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class PlayersProfileProfile(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    ongoingmatch: "MatchmakingClientReserve" = betterproto.message_field(2)
    global_stats: "GlobalStatistics" = betterproto.message_field(3)
    penalty_seconds: int = betterproto.uint32_field(4)
    penalty_reason: int = betterproto.uint32_field(5)
    vac_banned: int = betterproto.int32_field(6)
    ranking: "PlayerRankingInfo" = betterproto.message_field(7)
    commendation: "PlayerCommendationInfo" = betterproto.message_field(8)
    medals: "PlayerMedalsInfo" = betterproto.message_field(9)
    my_current_event: "TournamentEvent" = betterproto.message_field(10)
    my_current_event_teams: "list[TournamentTeam]" = betterproto.message_field(11)
    my_current_team: "TournamentTeam" = betterproto.message_field(12)
    my_current_event_stages: "list[TournamentEvent]" = betterproto.message_field(13)
    survey_vote: int = betterproto.uint32_field(14)
    activity: "AccountActivity" = betterproto.message_field(15)
    player_level: int = betterproto.int32_field(17)
    player_cur_xp: int = betterproto.int32_field(18)
    player_xp_bonus_flags: int = betterproto.int32_field(19)
    rankings: "list[PlayerRankingInfo]" = betterproto.message_field(20)


class PlayersProfile(GCProtobufMessage, msg=EMsg.PlayersProfile):
    request_id: int = betterproto.uint32_field(1)
    account_profiles: "list[PlayersProfileProfile]" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class PlayerOverwatchCaseUpdate(betterproto.Message):
    caseid: int = betterproto.uint64_field(1)
    suspectid: int = betterproto.uint32_field(3)
    fractionid: int = betterproto.uint32_field(4)
    rpt_aimbot: int = betterproto.uint32_field(5)
    rpt_wallhack: int = betterproto.uint32_field(6)
    rpt_speedhack: int = betterproto.uint32_field(7)
    rpt_teamharm: int = betterproto.uint32_field(8)
    reason: int = betterproto.uint32_field(9)


@dataclass(eq=False, repr=False)
class PlayerOverwatchCaseAssignment(betterproto.Message):
    caseid: int = betterproto.uint64_field(1)
    caseurl: str = betterproto.string_field(2)
    verdict: int = betterproto.uint32_field(3)
    timestamp: int = betterproto.uint32_field(4)
    throttleseconds: int = betterproto.uint32_field(5)
    suspectid: int = betterproto.uint32_field(6)
    fractionid: int = betterproto.uint32_field(7)
    numrounds: int = betterproto.uint32_field(8)
    fractionrounds: int = betterproto.uint32_field(9)
    streakconvictions: int = betterproto.int32_field(10)
    reason: int = betterproto.uint32_field(11)


@dataclass(eq=False, repr=False)
class PlayerOverwatchCaseStatus(betterproto.Message):
    caseid: int = betterproto.uint64_field(1)
    statusid: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CClientHeaderOverwatchEvidence(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    caseid: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class ClientTextMsg(betterproto.Message):
    id: int = betterproto.uint32_field(1)
    type: int = betterproto.uint32_field(2)
    payload: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class Client2GcTextMsg(betterproto.Message):
    id: int = betterproto.uint32_field(1)
    args: list[bytes] = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class MatchEndRunRewardDrops(betterproto.Message):
    serverinfo: "MatchmakingServerReservationResponse" = betterproto.message_field(3)
    match_end_quest_data: "GcServerQuestUpdateData" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class PreviewDataBlock(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    itemid: int = betterproto.uint64_field(2)
    defindex: int = betterproto.uint32_field(3)
    paintindex: int = betterproto.uint32_field(4)
    rarity: int = betterproto.uint32_field(5)
    quality: int = betterproto.uint32_field(6)
    paintwear: int = betterproto.uint32_field(7)
    paintseed: int = betterproto.uint32_field(8)
    killeaterscoretype: Union[int, None] = betterproto.uint32_field(9, optional=True, group="_killeaterscoretype")
    killeatervalue: Union[int, None] = betterproto.uint32_field(10, optional=True, group="_killeatervalue")
    customname: str = betterproto.string_field(11)
    stickers: "list[PreviewDataBlockSticker]" = betterproto.message_field(12)
    inventory: int = betterproto.uint32_field(13)
    origin: int = betterproto.uint32_field(14)
    questid: int = betterproto.uint32_field(15)
    dropreason: int = betterproto.uint32_field(16)
    musicindex: int = betterproto.uint32_field(17)
    entindex: int = betterproto.int32_field(18)


@dataclass(eq=False, repr=False)
class PreviewDataBlockSticker(betterproto.Message):
    slot: int = betterproto.uint32_field(1)
    sticker_id: int = betterproto.uint32_field(2)
    wear: float = betterproto.float_field(3)
    scale: float = betterproto.float_field(4)
    rotation: float = betterproto.float_field(5)
    tint_id: int = betterproto.uint32_field(6)
    offset_x: float = betterproto.float_field(7)
    offset_y: float = betterproto.float_field(8)


@dataclass(eq=False, repr=False)
class MatchEndRewardDropsNotification(betterproto.Message):
    iteminfo: "PreviewDataBlock" = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class ItemAcknowledged(betterproto.Message):
    iteminfo: "PreviewDataBlock" = betterproto.message_field(1)


class Client2GcEconPreviewDataBlockRequest(GCProtobufMessage, msg=EMsg.Client2GCEconPreviewDataBlockRequest):
    param_s: int = betterproto.uint64_field(1)
    param_a: int = betterproto.uint64_field(2)
    param_d: int = betterproto.uint64_field(3)
    param_m: int = betterproto.uint64_field(4)


class Client2GcEconPreviewDataBlockResponse(GCProtobufMessage, msg=EMsg.Client2GCEconPreviewDataBlockResponse):
    iteminfo: "PreviewDataBlock" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class MatchListRequestCurrentLiveGames(betterproto.Message):
    pass


class MatchListRequestLiveGameForUser(GCProtobufMessage, msg=EMsg.MatchListRequestLiveGameForUser):
    accountid: int = betterproto.uint32_field(1)


class MatchListRequestRecentUserGames(GCProtobufMessage, msg=EMsg.MatchListRequestRecentUserGames):
    accountid: int = betterproto.uint32_field(1)


class MatchListRequestTournamentGames(GCProtobufMessage, msg=EMsg.MatchListRequestTournamentGames):
    eventid: int = betterproto.int32_field(1)


class MatchListRequestFullGameInfo(GCProtobufMessage, msg=EMsg.MatchListRequestFullGameInfo):
    matchid: int = betterproto.uint64_field(1)
    outcomeid: int = betterproto.uint64_field(2)
    token: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class MatchInfo(betterproto.Message):
    matchid: int = betterproto.uint64_field(1)
    matchtime: int = betterproto.uint32_field(2)
    watchablematchinfo: "WatchableMatchInfo" = betterproto.message_field(3)
    roundstats_legacy: "MatchmakingServerRoundStats" = betterproto.message_field(4)
    roundstatsall: "list[MatchmakingServerRoundStats]" = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class TournamentGroupTeam(betterproto.Message):
    team_id: int = betterproto.int32_field(1)
    score: int = betterproto.int32_field(2)
    correctpick: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class TournamentGroup(betterproto.Message):
    groupid: int = betterproto.uint32_field(1)
    name: str = betterproto.string_field(2)
    desc: str = betterproto.string_field(3)
    picks_deprecated: int = betterproto.uint32_field(4)
    teams: "list[TournamentGroupTeam]" = betterproto.message_field(5)
    stage_ids: list[int] = betterproto.int32_field(6)
    picklockuntiltime: int = betterproto.uint32_field(7)
    pickableteams: int = betterproto.uint32_field(8)
    points_per_pick: int = betterproto.uint32_field(9)
    picks: "list[TournamentGroupPicks]" = betterproto.message_field(10)


@dataclass(eq=False, repr=False)
class TournamentGroupPicks(betterproto.Message):
    pickids: list[int] = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class TournamentSection(betterproto.Message):
    sectionid: int = betterproto.uint32_field(1)
    name: str = betterproto.string_field(2)
    desc: str = betterproto.string_field(3)
    groups: "list[TournamentGroup]" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class TournamentInfo(betterproto.Message):
    sections: "list[TournamentSection]" = betterproto.message_field(1)
    tournament_event: "TournamentEvent" = betterproto.message_field(2)
    tournament_teams: "list[TournamentTeam]" = betterproto.message_field(3)


class MatchList(GCProtobufMessage, msg=EMsg.MatchList):
    msgrequestid: int = betterproto.uint32_field(1)
    accountid: int = betterproto.uint32_field(2)
    servertime: int = betterproto.uint32_field(3)
    matches: "list[MatchInfo]" = betterproto.message_field(4)
    streams: "list[TournamentTeam]" = betterproto.message_field(5)
    tournamentinfo: "TournamentInfo" = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class Predictions(betterproto.Message):
    event_id: int = betterproto.uint32_field(1)
    group_match_team_picks: "list[PredictionsGroupMatchTeamPick]" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class PredictionsGroupMatchTeamPick(betterproto.Message):
    sectionid: int = betterproto.int32_field(1)
    groupid: int = betterproto.int32_field(2)
    index: int = betterproto.int32_field(3)
    teamid: int = betterproto.int32_field(4)
    itemid: int = betterproto.uint64_field(5)


@dataclass(eq=False, repr=False)
class Fantasy(betterproto.Message):
    event_id: int = betterproto.uint32_field(1)
    teams: "list[FantasyFantasyTeam]" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class FantasyFantasySlot(betterproto.Message):
    type: int = betterproto.int32_field(1)
    pick: int = betterproto.int32_field(2)
    itemid: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class FantasyFantasyTeam(betterproto.Message):
    sectionid: int = betterproto.int32_field(1)
    slots: "list[FantasyFantasySlot]" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CAttributeString(betterproto.Message):
    value: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class CStrike15Welcome(betterproto.Message):
    store_item_hash: int = betterproto.uint32_field(5)
    timeplayedconsecutively: int = betterproto.uint32_field(6)
    time_first_played: int = betterproto.uint32_field(10)
    last_time_played: int = betterproto.uint32_field(12)
    last_ip_address: int = betterproto.uint32_field(13)
    gscookieid: int = betterproto.uint64_field(18)
    uniqueid: int = betterproto.uint64_field(19)


@dataclass(eq=False, repr=False)
class ClientVarValueNotificationInfo(betterproto.Message):
    value_name: str = betterproto.string_field(1)
    value_int: int = betterproto.int32_field(2)
    server_addr: int = betterproto.uint32_field(3)
    server_port: int = betterproto.uint32_field(4)
    choked_blocks: list[str] = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class ServerVarValueNotificationInfo(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    viewangles: list[int] = betterproto.uint32_field(2)
    type: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class GiftsLeaderboardRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class GiftsLeaderboardResponse(betterproto.Message):
    servertime: int = betterproto.uint32_field(1)
    time_period_seconds: int = betterproto.uint32_field(2)
    total_gifts_given: int = betterproto.uint32_field(3)
    total_givers: int = betterproto.uint32_field(4)
    entries: "list[GiftsLeaderboardResponseGiftLeaderboardEntry]" = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class GiftsLeaderboardResponseGiftLeaderboardEntry(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    gifts: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class ClientSubmitSurveyVote(betterproto.Message):
    survey_id: int = betterproto.uint32_field(1)
    vote: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class Server2GcClientValidate(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class ClientTournamentInfo(betterproto.Message):
    eventid: int = betterproto.uint32_field(1)
    stageid: int = betterproto.uint32_field(2)
    game_type: int = betterproto.uint32_field(3)
    teamids: list[int] = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class EconCoupon(betterproto.Message):
    entryid: int = betterproto.uint32_field(1)
    defidx: int = betterproto.uint32_field(2)
    expiration_date: int = betterproto.fixed32_field(3)


@dataclass(eq=False, repr=False)
class QuestProgress(betterproto.Message):
    questid: int = betterproto.uint32_field(1)
    points_remaining: int = betterproto.uint32_field(2)
    bonus_points: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class AccountSeasonalOperation(betterproto.Message):
    season_value: int = betterproto.uint32_field(1)
    tier_unlocked: int = betterproto.uint32_field(2)
    premium_tiers: int = betterproto.uint32_field(3)
    mission_id: int = betterproto.uint32_field(4)
    missions_completed: int = betterproto.uint32_field(5)
    redeemable_balance: int = betterproto.uint32_field(6)
    season_pass_time: int = betterproto.uint32_field(7)


@dataclass(eq=False, repr=False)
class PersonaDataPublic(betterproto.Message):
    player_level: int = betterproto.int32_field(1)
    commendation: "PlayerCommendationInfo" = betterproto.message_field(2)
    elevated_state: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class GcGlobalGameSubscribe(betterproto.Message):
    ticket: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class GcGlobalGameUnsubscribe(betterproto.Message):
    timeleft: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class GcGlobalGamePlay(betterproto.Message):
    ticket: int = betterproto.uint64_field(1)
    gametimems: int = betterproto.uint32_field(2)
    msperpoint: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class AcknowledgePenalty(betterproto.Message):
    acknowledged: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class Client2GcRequestPrestigeCoin(betterproto.Message):
    defindex: int = betterproto.uint32_field(1)
    upgradeid: int = betterproto.uint64_field(2)
    hours: int = betterproto.uint32_field(3)
    prestigetime: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class Client2GcStreamUnlock(betterproto.Message):
    ticket: int = betterproto.uint64_field(1)
    os: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class ClientToGcRequestElevate(betterproto.Message):
    stage: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class ClientToGcChat(betterproto.Message):
    match_id: int = betterproto.uint64_field(1)
    text: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class GcToClientChat(betterproto.Message):
    account_id: int = betterproto.uint32_field(1)
    text: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ClientAuthKeyCode(betterproto.Message):
    eventid: int = betterproto.uint32_field(1)
    code: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class StrikeGotvSyncPacket(betterproto.Message):
    data: "GotvSyncPacket" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class PlayerDecalDigitalSignature(betterproto.Message):
    signature: bytes = betterproto.bytes_field(1)
    accountid: int = betterproto.uint32_field(2)
    rtime: int = betterproto.uint32_field(3)
    endpos: list[float] = betterproto.float_field(4)
    startpos: list[float] = betterproto.float_field(5)
    right: list[float] = betterproto.float_field(6)
    tx_defidx: int = betterproto.uint32_field(7)
    entindex: int = betterproto.int32_field(8)
    hitbox: int = betterproto.uint32_field(9)
    creationtime: float = betterproto.float_field(10)
    equipslot: int = betterproto.uint32_field(11)
    trace_id: int = betterproto.uint32_field(12)
    normal: list[float] = betterproto.float_field(13)
    tint_id: int = betterproto.uint32_field(14)


@dataclass(eq=False, repr=False)
class ClientPlayerDecalSign(betterproto.Message):
    data: "PlayerDecalDigitalSignature" = betterproto.message_field(1)
    itemid: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class ClientLogonFatalError(betterproto.Message):
    errorcode: int = betterproto.uint32_field(1)
    message: str = betterproto.string_field(2)
    country: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class ClientPollState(betterproto.Message):
    pollid: int = betterproto.uint32_field(1)
    names: list[str] = betterproto.string_field(2)
    values: list[int] = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class PartyRegister(betterproto.Message):
    id: int = betterproto.uint32_field(1)
    ver: int = betterproto.uint32_field(2)
    apr: int = betterproto.uint32_field(3)
    ark: int = betterproto.uint32_field(4)
    nby: int = betterproto.uint32_field(5)
    grp: int = betterproto.uint32_field(6)
    slots: int = betterproto.uint32_field(7)
    launcher: int = betterproto.uint32_field(8)
    game_type: int = betterproto.uint32_field(9)


@dataclass(eq=False, repr=False)
class PartySearch(betterproto.Message):
    ver: int = betterproto.uint32_field(1)
    apr: int = betterproto.uint32_field(2)
    ark: int = betterproto.uint32_field(3)
    grps: list[int] = betterproto.uint32_field(4)
    launcher: int = betterproto.uint32_field(5)
    game_type: int = betterproto.uint32_field(6)


@dataclass(eq=False, repr=False)
class PartySearchResults(betterproto.Message):
    entries: "list[PartySearchResultsEntry]" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class PartySearchResultsEntry(betterproto.Message):
    id: int = betterproto.uint32_field(1)
    grp: int = betterproto.uint32_field(2)
    game_type: int = betterproto.uint32_field(3)
    apr: int = betterproto.uint32_field(4)
    ark: int = betterproto.uint32_field(5)
    loc: int = betterproto.uint32_field(6)


@dataclass(eq=False, repr=False)
class PartyInvite(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    lobbyid: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class AccountRequestCoPlays(betterproto.Message):
    players: "list[AccountRequestCoPlaysPlayer]" = betterproto.message_field(1)
    servertime: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class AccountRequestCoPlaysPlayer(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    rtcoplay: int = betterproto.uint32_field(2)
    online: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class ClientToGcRequestTicket(betterproto.Message):
    authorized_steam_id: int = betterproto.fixed64_field(1)
    authorized_public_ip: int = betterproto.fixed32_field(2)
    gameserver_steam_id: int = betterproto.fixed64_field(3)
    gameserver_sdr_routing: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class GcToClientSteamDatagramTicket(betterproto.Message):
    serialized_ticket: bytes = betterproto.bytes_field(16)


@dataclass(eq=False, repr=False)
class ClientRequestOffers(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ClientRequestSouvenir(betterproto.Message):
    itemid: int = betterproto.uint64_field(1)
    matchid: int = betterproto.uint64_field(2)
    eventid: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class ClientAccountBalance(betterproto.Message):
    amount: int = betterproto.uint64_field(1)
    url: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ClientPartyJoinRelay(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    lobbyid: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class ClientPartyWarning(betterproto.Message):
    entries: "list[ClientPartyWarningEntry]" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ClientPartyWarningEntry(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    warntype: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class SetEventFavorite(betterproto.Message):
    eventid: int = betterproto.uint64_field(1)
    is_favorite: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class GetEventFavoritesRequest(betterproto.Message):
    all_events: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class GetEventFavoritesResponse(betterproto.Message):
    all_events: bool = betterproto.bool_field(1)
    json_favorites: str = betterproto.string_field(2)
    json_featured: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class ClientPerfReport(betterproto.Message):
    entries: "list[ClientPerfReportEntry]" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ClientPerfReportEntry(betterproto.Message):
    perfcounter: int = betterproto.uint32_field(1)
    length: int = betterproto.uint32_field(2)
    reference: bytes = betterproto.bytes_field(3)
    actual: bytes = betterproto.bytes_field(4)
    sourceid: int = betterproto.uint32_field(5)
    status: int = betterproto.uint32_field(6)


@dataclass(eq=False, repr=False)
class CvDiagnostic(betterproto.Message):
    id: int = betterproto.uint32_field(1)
    extended: int = betterproto.uint32_field(2)
    value: int = betterproto.uint64_field(3)
    string_value: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class ClientReportValidation(betterproto.Message):
    file_report: str = betterproto.string_field(1)
    command_line: str = betterproto.string_field(2)
    total_files: int = betterproto.uint32_field(3)
    internal_error: int = betterproto.uint32_field(4)
    trust_time: int = betterproto.uint32_field(5)
    count_pending: int = betterproto.uint32_field(6)
    count_completed: int = betterproto.uint32_field(7)
    process_id: int = betterproto.uint32_field(8)
    osversion: int = betterproto.int32_field(9)
    clientreportversion: int = betterproto.uint32_field(10)
    status_id: int = betterproto.uint32_field(11)
    diagnostic1: int = betterproto.uint32_field(12)
    diagnostic2: int = betterproto.uint64_field(13)
    diagnostic3: int = betterproto.uint64_field(14)
    last_launch_data: str = betterproto.string_field(15)
    report_count: int = betterproto.uint32_field(16)
    client_time: int = betterproto.uint64_field(17)
    diagnostic4: int = betterproto.uint64_field(18)
    diagnostic5: int = betterproto.uint64_field(19)
    diagnostics: "list[CvDiagnostic]" = betterproto.message_field(20)


@dataclass(eq=False, repr=False)
class ClientRefuseSecureMode(betterproto.Message):
    file_report: str = betterproto.string_field(1)
    offer_insecure_mode: bool = betterproto.bool_field(2)
    offer_secure_mode: bool = betterproto.bool_field(3)
    show_unsigned_ui: bool = betterproto.bool_field(4)
    kick_user: bool = betterproto.bool_field(5)
    show_trusted_ui: bool = betterproto.bool_field(6)
    show_warning_not_trusted: bool = betterproto.bool_field(7)


@dataclass(eq=False, repr=False)
class ClientRequestValidation(betterproto.Message):
    full_report: bool = betterproto.bool_field(1)
    module: str = betterproto.string_field(2)


class PremierSeasonSummary(GCProtobufMessage, msg=EMsg.PremierSeasonSummary):  # request
    account_id: int = betterproto.uint32_field(1)
    season_id: int = betterproto.uint32_field(2)


class PremierSeasonSummarySummary(GCProtobufMessage, msg=EMsg.PremierSeasonSummary):  # response
    account_id: int = betterproto.uint32_field(1)
    season_id: int = betterproto.uint32_field(2)
    data_per_week: "list[PremierSeasonSummaryDataPerWeek]" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class PremierSeasonSummaryDataPerWeek(betterproto.Message):
    week_id: int = betterproto.uint64_field(1)
    rank_id: int = betterproto.uint32_field(2)
    matches_played: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class PremierSeasonSummaryDataPerMap(betterproto.Message):
    map_id: int = betterproto.uint32_field(1)
    wins: int = betterproto.uint32_field(2)
    ties: int = betterproto.uint32_field(3)
    losses: int = betterproto.uint32_field(4)
    rounds: int = betterproto.uint32_field(5)
    kills: int = betterproto.uint32_field(6)
    headshots: int = betterproto.uint32_field(7)
    assists: int = betterproto.uint32_field(8)
    deaths: int = betterproto.uint32_field(9)
    mvps: int = betterproto.uint32_field(10)
    rounds_3k: int = betterproto.uint32_field(11)
    rounds_4k: int = betterproto.uint32_field(12)
    rounds_5k: int = betterproto.uint32_field(13)



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

# NB: this is the only types file that is expected to importable at runtime
#     these are internal types and user's shouldn't ever have to use them for the public API

from typing import (
    NewType as _NewType,
    SupportsIndex as _SupportsIndex,
    SupportsInt as _SupportsInt,
    TypeAlias as _TypeAlias,
)

Intable: _TypeAlias = _SupportsInt | _SupportsIndex | str | bytes  # anything int(x) wouldn't normally fail on
ID64 = _NewType("ID64", int)  # u64


AppID = _NewType("AppID", int)  # u32
ContextID = _NewType("ContextID", int)  # u64
PackageID = _NewType("PackageID", int)  # u32
BundleID = _NewType("BundleID", int)  # u32

AssetID = _NewType("AssetID", int)  # u64
ClassID = _NewType("ClassID", int)
InstanceID = _NewType("InstanceID", int)
CacheKey: _TypeAlias = tuple[ClassID, InstanceID]
TradeOfferID = _NewType("TradeOfferID", int)  # u64


class ChatGroupID(
    int
): ...  # technically a u64 but we only use 32 bits for now and Steam treats it as a u32 (Group.id is a u32)


ChatID = _NewType("ChatID", int)  # u64
RoleID = _NewType("RoleID", int)  # u32


# this relationship may seem a little weird but it makes type checking easier
class ID32(ChatGroupID):  # u32
    ...


PostID = _NewType("PostID", int)  # u64
CommentID = _NewType("CommentID", int)  # u64
PublishedFileID = _NewType("PublishedFileID", int)  # u64

ManifestID = _NewType("ManifestID", int)
DepotID = _NewType("DepotID", int)

LeaderboardID = _NewType("LeaderboardID", int)



from typing import Final

import betterproto

APP_ID: Final = 730

from ....protobufs.msg import GCProtobufMessage
from . import (
    base as base,
    cstrike as cstrike,
    econ as econ,
    engine as engine,
    sdk as sdk,
    struct_messages as struct_messages,
    system_messages as system_messages,
)

[setattr(cls, "_betterproto", betterproto.ProtoClassMetadata(cls)) for cls in GCProtobufMessage.__subclasses__()]



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_cloud.steamclient.proto
# plugin: python-betterproto
# Last updated 21/12/2022

from dataclasses import dataclass

import betterproto

from .msg import UnifiedMessage


class EProtoExecutionSite(betterproto.Enum):
    Unknown = 0
    SteamClient = 2


class EPublishedFileStorageSystem(betterproto.Enum):
    Invalid = 0
    LegacyCloud = 1
    Depot = 2
    UGCCloud = 3


class ECloudStoragePersistState(betterproto.Enum):
    Persisted = 0
    Forgotten = 1
    Deleted = 2


class ECloudPendingRemoteOperation(betterproto.Enum):
    NONE = 0
    AppSessionActive = 1
    UploadInProgress = 2
    UploadPending = 3
    AppSessionSuspended = 4


class ClientLogUploadCheckNotification(UnifiedMessage, um_name="Cloud.ClientLogUploadCheck"):
    client_id: int = betterproto.uint64_field(1)


class ClientLogUploadCompleteNotification(UnifiedMessage, um_name="Cloud.ClientLogUploadComplete"):
    client_id: int = betterproto.uint64_field(1)
    request_id: int = betterproto.uint64_field(2)


class GetUploadServerInfoRequest(UnifiedMessage, um_name="Cloud.GetUploadServerInfo"):
    appid: int = betterproto.uint32_field(1)


class GetUploadServerInfoResponse(UnifiedMessage, um_name="Cloud.GetUploadServerInfo"):
    server_url: str = betterproto.string_field(1)


class BeginHttpUploadRequest(UnifiedMessage, um_name="Cloud.BeginHTTPUpload"):
    appid: int = betterproto.uint32_field(1)
    file_size: int = betterproto.uint32_field(2)
    filename: str = betterproto.string_field(3)
    file_sha: str = betterproto.string_field(4)
    is_public: bool = betterproto.bool_field(5)
    platforms_to_sync: list[str] = betterproto.string_field(6)
    request_headers_names: list[str] = betterproto.string_field(7)
    request_headers_values: list[str] = betterproto.string_field(8)
    upload_batch_id: int = betterproto.uint64_field(9)


class BeginHttpUploadResponse(UnifiedMessage, um_name="Cloud.BeginHTTPUpload"):
    ugcid: int = betterproto.fixed64_field(1)
    timestamp: int = betterproto.fixed32_field(2)
    url_host: str = betterproto.string_field(3)
    url_path: str = betterproto.string_field(4)
    use_https: bool = betterproto.bool_field(5)
    request_headers: dict[str, str] = betterproto.map_field(6, betterproto.TYPE_STRING, betterproto.TYPE_STRING)


class CommitHttpUploadRequest(UnifiedMessage, um_name="Cloud.CommitHTTPUpload"):
    transfer_succeeded: bool = betterproto.bool_field(1)
    appid: int = betterproto.uint32_field(2)
    file_sha: str = betterproto.string_field(3)
    filename: str = betterproto.string_field(4)


class CommitHttpUploadResponse(UnifiedMessage, um_name="Cloud.CommitHTTPUpload"):
    file_committed: bool = betterproto.bool_field(1)


class BeginUgcUploadRequest(UnifiedMessage, um_name="Cloud.BeginUGCUpload"):
    appid: int = betterproto.uint32_field(1)
    file_size: int = betterproto.uint32_field(2)
    filename: str = betterproto.string_field(3)
    file_sha: str = betterproto.string_field(4)
    content_type: str = betterproto.string_field(5)


class BeginUgcUploadResponse(UnifiedMessage, um_name="Cloud.BeginUGCUpload"):
    storage_system: EPublishedFileStorageSystem = betterproto.enum_field(1)
    ugcid: int = betterproto.fixed64_field(2)
    timestamp: int = betterproto.fixed32_field(3)
    url_host: str = betterproto.string_field(4)
    url_path: str = betterproto.string_field(5)
    use_https: bool = betterproto.bool_field(6)
    request_headers: dict[str, str] = betterproto.map_field(7, betterproto.TYPE_STRING, betterproto.TYPE_STRING)


class CommitUgcUploadRequest(UnifiedMessage, um_name="Cloud.CommitUGCUpload"):
    transfer_succeeded: bool = betterproto.bool_field(1)
    appid: int = betterproto.uint32_field(2)
    ugcid: int = betterproto.fixed64_field(3)


class CommitUgcUploadResponse(UnifiedMessage, um_name="Cloud.CommitUGCUpload"):
    file_committed: bool = betterproto.bool_field(1)


class GetFileDetailsRequest(UnifiedMessage, um_name="Cloud.GetFileDetails"):
    ugcid: int = betterproto.uint64_field(1)
    appid: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class UserFile(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    ugcid: int = betterproto.uint64_field(2)
    filename: str = betterproto.string_field(3)
    timestamp: int = betterproto.uint64_field(4)
    file_size: int = betterproto.uint32_field(5)
    url: str = betterproto.string_field(6)
    steamid_creator: int = betterproto.fixed64_field(7)
    flags: int = betterproto.uint32_field(8)
    platforms_to_sync: list[str] = betterproto.string_field(9)
    file_sha: str = betterproto.string_field(10)


class GetFileDetailsResponse(UnifiedMessage, um_name="Cloud.GetFileDetails"):
    details: UserFile = betterproto.message_field(1)


class EnumerateUserFilesRequest(UnifiedMessage, um_name="Cloud.EnumerateUserFiles"):
    appid: int = betterproto.uint32_field(1)
    extended_details: bool = betterproto.bool_field(2)
    count: int = betterproto.uint32_field(3)
    start_index: int = betterproto.uint32_field(4)


class EnumerateUserFilesResponse(UnifiedMessage, um_name="Cloud.EnumerateUserFiles"):
    files: list[UserFile] = betterproto.message_field(1)
    total_files: int = betterproto.uint32_field(2)


class DeleteRequest(UnifiedMessage, um_name="Cloud.Delete"):
    filename: str = betterproto.string_field(1)
    appid: int = betterproto.uint32_field(2)
    upload_batch_id: int = betterproto.uint64_field(3)


class DeleteResponse(UnifiedMessage, um_name="Cloud.Delete"):
    pass


class GetClientEncryptionKeyRequest(UnifiedMessage, um_name="Cloud.GetClientEncryptionKey"):
    pass


class GetClientEncryptionKeyResponse(UnifiedMessage, um_name="Cloud.GetClientEncryptionKey"):
    key: bytes = betterproto.bytes_field(1)
    crc: int = betterproto.int32_field(2)


class CdnReportNotification(UnifiedMessage, um_name="Cloud.CDNReport"):
    steamid: int = betterproto.fixed64_field(1)
    url: str = betterproto.string_field(2)
    success: bool = betterproto.bool_field(3)
    http_status_code: int = betterproto.uint32_field(4)
    expected_bytes: int = betterproto.uint64_field(5)
    received_bytes: int = betterproto.uint64_field(6)
    duration: int = betterproto.uint32_field(7)


class ExternalStorageTransferReportNotification(UnifiedMessage, um_name="Cloud.ExternalStorageTransferReport"):
    host: str = betterproto.string_field(1)
    path: str = betterproto.string_field(2)
    is_upload: bool = betterproto.bool_field(3)
    success: bool = betterproto.bool_field(4)
    http_status_code: int = betterproto.uint32_field(5)
    bytes_expected: int = betterproto.uint64_field(6)
    bytes_actual: int = betterproto.uint64_field(7)
    duration_ms: int = betterproto.uint32_field(8)
    cellid: int = betterproto.uint32_field(9)
    proxied: bool = betterproto.bool_field(10)
    ipv6_local: bool = betterproto.bool_field(11)
    ipv6_remote: bool = betterproto.bool_field(12)
    time_to_connect_ms: int = betterproto.uint32_field(13)
    time_to_send_req_ms: int = betterproto.uint32_field(14)
    time_to_first_byte_ms: int = betterproto.uint32_field(15)
    time_to_last_byte_ms: int = betterproto.uint32_field(16)


class BeginAppUploadBatchRequest(UnifiedMessage, um_name="Cloud.BeginAppUploadBatch"):
    appid: int = betterproto.uint32_field(1)
    machine_name: str = betterproto.string_field(2)
    files_to_upload: list[str] = betterproto.string_field(3)
    files_to_delete: list[str] = betterproto.string_field(4)
    client_id: int = betterproto.uint64_field(5)
    app_build_id: int = betterproto.uint64_field(6)


class BeginAppUploadBatchResponse(UnifiedMessage, um_name="Cloud.BeginAppUploadBatch"):
    batch_id: int = betterproto.uint64_field(1)
    app_change_number: int = betterproto.uint64_field(4)


class CompleteAppUploadBatchNotification(UnifiedMessage, um_name="Cloud.CompleteAppUploadBatch"):
    appid: int = betterproto.uint32_field(1)
    batch_id: int = betterproto.uint64_field(2)
    batch_eresult: int = betterproto.uint32_field(3)


class CompleteAppUploadBatchRequest(UnifiedMessage, um_name="Cloud.CompleteAppUploadBatchBlocking"):
    appid: int = betterproto.uint32_field(1)
    batch_id: int = betterproto.uint64_field(2)
    batch_eresult: int = betterproto.uint32_field(3)


class CompleteAppUploadBatchResponse(UnifiedMessage, um_name="Cloud.CompleteAppUploadBatchBlocking"):
    pass


class ClientBeginFileUploadRequest(UnifiedMessage, um_name="Cloud.ClientBeginFileUpload"):
    appid: int = betterproto.uint32_field(1)
    file_size: int = betterproto.uint32_field(2)
    raw_file_size: int = betterproto.uint32_field(3)
    file_sha: bytes = betterproto.bytes_field(4)
    time_stamp: int = betterproto.uint64_field(5)
    filename: str = betterproto.string_field(6)
    platforms_to_sync: int = betterproto.uint32_field(7)
    cell_id: int = betterproto.uint32_field(9)
    can_encrypt: bool = betterproto.bool_field(10)
    is_shared_file: bool = betterproto.bool_field(11)
    deprecated_realm: int = betterproto.uint32_field(12)
    upload_batch_id: int = betterproto.uint64_field(13)


@dataclass(eq=False, repr=False)
class ClientCloudFileUploadBlockDetails(betterproto.Message):
    url_host: str = betterproto.string_field(1)
    url_path: str = betterproto.string_field(2)
    use_https: bool = betterproto.bool_field(3)
    http_method: int = betterproto.int32_field(4)
    request_headers: dict[str, str] = betterproto.map_field(5, betterproto.TYPE_STRING, betterproto.TYPE_STRING)
    block_offset: int = betterproto.uint64_field(6)
    block_length: int = betterproto.uint32_field(7)
    explicit_body_data: bytes = betterproto.bytes_field(8)
    may_parallelize: bool = betterproto.bool_field(9)


class ClientBeginFileUploadResponse(UnifiedMessage, um_name="Cloud.ClientBeginFileUpload"):
    encrypt_file: bool = betterproto.bool_field(1)
    block_requests: list[ClientCloudFileUploadBlockDetails] = betterproto.message_field(2)


class ClientCommitFileUploadRequest(UnifiedMessage, um_name="Cloud.ClientCommitFileUpload"):
    transfer_succeeded: bool = betterproto.bool_field(1)
    appid: int = betterproto.uint32_field(2)
    file_sha: bytes = betterproto.bytes_field(3)
    filename: str = betterproto.string_field(4)


class ClientCommitFileUploadResponse(UnifiedMessage, um_name="Cloud.ClientCommitFileUpload"):
    file_committed: bool = betterproto.bool_field(1)


class ClientFileDownloadRequest(UnifiedMessage, um_name="Cloud.ClientFileDownload"):
    appid: int = betterproto.uint32_field(1)
    filename: str = betterproto.string_field(2)
    realm: int = betterproto.uint32_field(3)


class ClientFileDownloadResponse(UnifiedMessage, um_name="Cloud.ClientFileDownload"):
    appid: int = betterproto.uint32_field(1)
    file_size: int = betterproto.uint32_field(2)
    raw_file_size: int = betterproto.uint32_field(3)
    sha_file: bytes = betterproto.bytes_field(4)
    time_stamp: int = betterproto.uint64_field(5)
    is_explicit_delete: bool = betterproto.bool_field(6)
    url_host: str = betterproto.string_field(7)
    url_path: str = betterproto.string_field(8)
    use_https: bool = betterproto.bool_field(9)
    request_headers: dict[str, str] = betterproto.map_field(10, betterproto.TYPE_STRING, betterproto.TYPE_STRING)
    encrypted: bool = betterproto.bool_field(11)


class ClientDeleteFileRequest(UnifiedMessage, um_name="Cloud.ClientDeleteFile"):
    appid: int = betterproto.uint32_field(1)
    filename: str = betterproto.string_field(2)
    is_explicit_delete: bool = betterproto.bool_field(3)
    upload_batch_id: int = betterproto.uint64_field(4)


class ClientDeleteFileResponse(UnifiedMessage, um_name="Cloud.ClientDeleteFile"):
    pass


class ClientConflictResolutionNotification(UnifiedMessage, um_name="Cloud.ClientConflictResolution"):
    appid: int = betterproto.uint32_field(1)
    chose_local_files: bool = betterproto.bool_field(2)


class EnumerateUserAppsRequest(UnifiedMessage, um_name="Cloud.EnumerateUserApps"):
    pass


class EnumerateUserAppsResponse(UnifiedMessage, um_name="Cloud.EnumerateUserApps"):
    apps: "list[EnumerateUserAppsResponseApps]" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class EnumerateUserAppsResponseApps(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    totalcount: int = betterproto.int32_field(2)
    totalsize: int = betterproto.int64_field(3)


class GetAppFileChangelistRequest(UnifiedMessage, um_name="Cloud.GetAppFileChangelist"):
    appid: int = betterproto.uint32_field(1)
    synced_change_number: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class AppFileInfo(betterproto.Message):
    file_name: str = betterproto.string_field(1)
    sha_file: bytes = betterproto.bytes_field(2)
    time_stamp: int = betterproto.uint64_field(3)
    raw_file_size: int = betterproto.uint32_field(4)
    persist_state: ECloudStoragePersistState = betterproto.enum_field(5)
    platforms_to_sync: int = betterproto.uint32_field(6)
    path_prefix_index: int = betterproto.uint32_field(7)
    machine_name_index: int = betterproto.uint32_field(8)


class GetAppFileChangelistResponse(UnifiedMessage, um_name="Cloud.GetAppFileChangelist"):
    current_change_number: int = betterproto.uint64_field(1)
    files: list[AppFileInfo] = betterproto.message_field(2)
    is_only_delta: bool = betterproto.bool_field(3)
    path_prefixes: list[str] = betterproto.string_field(4)
    machine_names: list[str] = betterproto.string_field(5)
    app_buildid_hwm: int = betterproto.uint64_field(6)


class AppSessionSuspendRequest(UnifiedMessage, um_name="Cloud.SuspendAppSession"):
    appid: int = betterproto.uint32_field(1)
    client_id: int = betterproto.uint64_field(2)
    machine_name: str = betterproto.string_field(3)
    cloud_sync_completed: bool = betterproto.bool_field(4)


class AppSessionSuspendResponse(UnifiedMessage, um_name="Cloud.SuspendAppSession"):
    pass


class AppSessionResumeRequest(UnifiedMessage, um_name="Cloud.ResumeAppSession"):
    appid: int = betterproto.uint32_field(1)
    client_id: int = betterproto.uint64_field(2)


class AppSessionResumeResponse(UnifiedMessage, um_name="Cloud.ResumeAppSession"):
    pass


class AppLaunchIntentRequest(UnifiedMessage, um_name="Cloud.AppLaunchIntent"):
    appid: int = betterproto.uint32_field(1)
    client_id: int = betterproto.uint64_field(2)
    machine_name: str = betterproto.string_field(3)
    ignore_pending_operations: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class PendingRemoteOperation(betterproto.Message):
    operation: ECloudPendingRemoteOperation = betterproto.enum_field(1)
    machine_name: str = betterproto.string_field(2)
    client_id: int = betterproto.uint64_field(3)
    time_last_updated: int = betterproto.uint32_field(4)


class AppLaunchIntentResponse(UnifiedMessage, um_name="Cloud.SignalAppLaunchIntent"):
    pending_remote_operations: list[PendingRemoteOperation] = betterproto.message_field(1)


class AppExitSyncDoneNotification(UnifiedMessage, um_name="Cloud.SignalAppExitSyncDone"):
    appid: int = betterproto.uint32_field(1)
    client_id: int = betterproto.uint64_field(2)
    uploads_completed: bool = betterproto.bool_field(3)


class ClientGetAppQuotaUsageRequest(UnifiedMessage, um_name="Cloud.ClientGetAppQuotaUsage"):
    appid: int = betterproto.uint32_field(1)


class ClientGetAppQuotaUsageResponse(UnifiedMessage, um_name="Cloud.ClientGetAppQuotaUsage"):
    existing_files: int = betterproto.uint32_field(1)
    existing_bytes: int = betterproto.uint64_field(2)
    max_num_files: int = betterproto.uint32_field(3)
    max_num_bytes: int = betterproto.uint64_field(4)


class AppCloudStateChangeNotification(
    UnifiedMessage, um_name="CloudClient.NotifyAppStateChange"
):  # TODO can this be used for update detection?
    appid: int = betterproto.uint32_field(1)
    app_change_number: int = betterproto.uint64_field(2)


class ClientLogUploadRequestNotification(UnifiedMessage, um_name="CloudClient.ClientLogUpload"):
    request_id: int = betterproto.uint64_field(1)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_auth.steamclient.proto
# plugin: python-betterproto
# Last updated 02/09/2022

from dataclasses import dataclass

import betterproto

from .base import CMsgIpAddress
from .emsg import EMsg
from .msg import UnifiedMessage


class EAuthTokenPlatformType(betterproto.Enum):
    Unknown = 0
    SteamClient = 1
    WebBrowser = 2
    MobileApp = 3


class EAuthSessionGuardType(betterproto.Enum):
    Unknown = 0
    NONE = 1
    EmailCode = 2
    DeviceCode = 3
    DeviceConfirmation = 4
    EmailConfirmation = 5
    MachineToken = 6


class EAuthSessionSecurityHistory(betterproto.Enum):
    Invalid = 0
    UsedPreviously = 1
    NoPriorHistory = 2


class EAuthTokenState(betterproto.Enum):
    Invalid = 0
    New = 1
    Confirmed = 2
    Issued = 3
    Denied = 4
    LoggedOut = 5
    Revoked = 99


class ESessionPersistence(betterproto.Enum):
    Invalid = -1
    Ephemeral = 0
    Persistent = 1


class GenerateAccessTokenForAppRequest(UnifiedMessage, um_name="Authentication.GenerateAccessTokenForApp"):
    refresh_token: str = betterproto.string_field(1)
    steamid: int = betterproto.fixed64_field(2)


class GenerateAccessTokenForAppResponse(UnifiedMessage, um_name="Authentication.GenerateAccessTokenForApp"):
    access_token: str = betterproto.string_field(1)
    refresh_token: str = betterproto.string_field(2)


class GetPasswordRsaPublicKeyRequest(
    UnifiedMessage,
    msg=EMsg.ServiceMethodCallFromClientNonAuthed,
    um_name="Authentication.GetPasswordRSAPublicKey",
):
    account_name: str = betterproto.string_field(1)


class GetPasswordRsaPublicKeyResponse(UnifiedMessage, um_name="Authentication.GetPasswordRSAPublicKey"):
    publickey_mod: str = betterproto.string_field(1)
    publickey_exp: str = betterproto.string_field(2)
    timestamp: int = betterproto.uint64_field(3)


class BeginAuthSessionViaQrRequest(UnifiedMessage, um_name="Authentication.BeginAuthSessionViaQR"):
    device_friendly_name: str = betterproto.string_field(1)
    platform_type: EAuthTokenPlatformType = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class AllowedConfirmation(betterproto.Message):
    confirmation_type: EAuthSessionGuardType = betterproto.enum_field(1)
    associated_message: str = betterproto.string_field(2)


class BeginAuthSessionViaQrResponse(UnifiedMessage, um_name="Authentication.BeginAuthSessionViaQR"):
    client_id: int = betterproto.uint64_field(1)
    challenge_url: str = betterproto.string_field(2)
    request_id: bytes = betterproto.bytes_field(3)
    interval: float = betterproto.float_field(4)
    allowed_confirmations: list[AllowedConfirmation] = betterproto.message_field(5)
    version: int = betterproto.int32_field(6)


class BeginAuthSessionViaCredentialsRequest(
    UnifiedMessage,
    msg=EMsg.ServiceMethodCallFromClientNonAuthed,
    um_name="Authentication.BeginAuthSessionViaCredentials",
):
    device_friendly_name: str = betterproto.string_field(1)
    account_name: str = betterproto.string_field(2)
    encrypted_password: str = betterproto.string_field(3)
    encryption_timestamp: int = betterproto.uint64_field(4)
    platform_type: EAuthTokenPlatformType = betterproto.enum_field(6)
    persistence: ESessionPersistence = betterproto.enum_field(7)
    website_id: str = betterproto.string_field(8)


class BeginAuthSessionViaCredentialsResponse(UnifiedMessage, um_name="Authentication.BeginAuthSessionViaCredentials"):
    client_id: int = betterproto.uint64_field(1)
    request_id: bytes = betterproto.bytes_field(2)
    interval: float = betterproto.float_field(3)
    allowed_confirmations: list[AllowedConfirmation] = betterproto.message_field(4)
    steamid: int = betterproto.uint64_field(5)
    weak_token: str = betterproto.string_field(6)


class PollAuthSessionStatusRequest(
    UnifiedMessage, um_name="Authentication.PollAuthSessionStatus", msg=EMsg.ServiceMethodCallFromClientNonAuthed
):
    client_id: int = betterproto.uint64_field(1)
    request_id: bytes = betterproto.bytes_field(2)
    token_to_revoke: int = betterproto.fixed64_field(3)


class PollAuthSessionStatusResponse(UnifiedMessage, um_name="Authentication.PollAuthSessionStatus"):
    new_client_id: int = betterproto.uint64_field(1)
    new_challenge_url: str = betterproto.string_field(2)
    refresh_token: str = betterproto.string_field(3)
    access_token: str = betterproto.string_field(4)
    had_remote_interaction: bool = betterproto.bool_field(5)
    account_name: str = betterproto.string_field(6)


class GetAuthSessionInfoRequest(UnifiedMessage, um_name="Authentication.GetAuthSessionInfo"):
    client_id: int = betterproto.uint64_field(1)


class GetAuthSessionInfoResponse(UnifiedMessage, um_name="Authentication.GetAuthSessionInfo"):
    ip: str = betterproto.string_field(1)
    geoloc: str = betterproto.string_field(2)
    city: str = betterproto.string_field(3)
    state: str = betterproto.string_field(4)
    country: str = betterproto.string_field(5)
    platform_type: EAuthTokenPlatformType = betterproto.enum_field(6)
    device_friendly_name: str = betterproto.string_field(7)
    version: int = betterproto.int32_field(8)
    login_history: EAuthSessionSecurityHistory = betterproto.enum_field(9)
    requestor_location_mismatch: bool = betterproto.bool_field(10)
    high_usage_login: bool = betterproto.bool_field(11)
    requested_persistence: ESessionPersistence = betterproto.enum_field(12)


class UpdateAuthSessionWithMobileConfirmationRequest(
    UnifiedMessage, um_name="Authentication.UpdateAuthSessionWithMobileConfirmation"
):
    version: int = betterproto.int32_field(1)
    client_id: int = betterproto.uint64_field(2)
    steamid: int = betterproto.fixed64_field(3)
    signature: bytes = betterproto.bytes_field(4)
    confirm: bool = betterproto.bool_field(5)
    persistence: ESessionPersistence = betterproto.enum_field(6)


class UpdateAuthSessionWithMobileConfirmationResponse(
    UnifiedMessage, um_name="Authentication.UpdateAuthSessionWithMobileConfirmation"
):
    pass


class UpdateAuthSessionWithSteamGuardCodeRequest(
    UnifiedMessage,
    um_name="Authentication.UpdateAuthSessionWithSteamGuardCode",
    msg=EMsg.ServiceMethodCallFromClientNonAuthed,
):
    client_id: int = betterproto.uint64_field(1)
    steamid: int = betterproto.fixed64_field(2)
    code: str = betterproto.string_field(3)
    code_type: EAuthSessionGuardType = betterproto.enum_field(4)


class UpdateAuthSessionWithSteamGuardCodeResponse(
    UnifiedMessage, um_name="Authentication.UpdateAuthSessionWithSteamGuardCode"
):
    pass


class AccessTokenGenerateForAppRequest(UnifiedMessage, um_name="Authentication.AccessTokenGenerateForApp"):
    refresh_token: str = betterproto.string_field(1)
    steamid: int = betterproto.fixed64_field(2)


class AccessTokenGenerateForAppResponse(UnifiedMessage, um_name="Authentication.AccessTokenGenerateForApp"):
    access_token: str = betterproto.string_field(1)


class GetAuthSessionsForAccountRequest(UnifiedMessage, um_name="Authentication.GetAuthSessionsForAccount"):
    pass


class GetAuthSessionsForAccountResponse(UnifiedMessage, um_name="Authentication.GetAuthSessionsForAccount"):
    client_ids: list[int] = betterproto.uint64_field(1)


class MigrateMobileSessionRequest(UnifiedMessage, um_name="Authentication.MigrateMobileSession"):
    steamid: int = betterproto.fixed64_field(1)
    token: str = betterproto.string_field(2)
    signature: str = betterproto.string_field(3)


class MigrateMobileSessionResponse(UnifiedMessage, um_name="Authentication.MigrateMobileSession"):
    refresh_token: str = betterproto.string_field(1)
    access_token: str = betterproto.string_field(2)


class SupportQueryRefreshTokensByAccountRequest(
    UnifiedMessage, um_name="AuthenticationSupport.QueryRefreshTokensByAccount"
):
    steamid: int = betterproto.fixed64_field(1)
    include_revoked_tokens: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class RefreshTokenDescription(betterproto.Message):
    token_id: int = betterproto.fixed64_field(1)
    token_description: str = betterproto.string_field(2)
    time_updated: int = betterproto.uint32_field(3)
    platform_type: EAuthTokenPlatformType = betterproto.enum_field(4)
    token_state: EAuthTokenState = betterproto.enum_field(5)
    owner_steamid: int = betterproto.fixed64_field(6)


class QueryRefreshTokensByAccountResponse(UnifiedMessage, um_name="AuthenticationSupport.QueryRefreshTokensByAccount"):
    refresh_tokens: list[RefreshTokenDescription] = betterproto.message_field(1)


class QueryRefreshTokenByIdRequest(UnifiedMessage, um_name="AuthenticationSupport.QueryRefreshTokenByID"):
    token_id: int = betterproto.fixed64_field(1)


class QueryRefreshTokenByIdResponse(UnifiedMessage, um_name="AuthenticationSupport.QueryRefreshTokenByID"):
    refresh_tokens: list[RefreshTokenDescription] = betterproto.message_field(1)


class RevokeTokenRequest(UnifiedMessage, um_name="AuthenticationSupport.RevokeToken"):
    token_id: int = betterproto.fixed64_field(1)


class RevokeTokenResponse(UnifiedMessage, um_name="AuthenticationSupport.RevokeToken"):
    pass


class GetTokenHistoryRequest(UnifiedMessage, um_name="AuthenticationSupport.GetTokenHistory"):
    token_id: int = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class RefreshTokenAudit(betterproto.Message):
    action: int = betterproto.int32_field(1)
    time: int = betterproto.uint32_field(2)
    ip: CMsgIpAddress = betterproto.message_field(3)
    actor: int = betterproto.fixed64_field(4)


class GetTokenHistoryResponse(UnifiedMessage, um_name="AuthenticationSupport.GetTokenHistory"):
    history: list[RefreshTokenAudit] = betterproto.message_field(1)


class CreateNonceRequest(UnifiedMessage, um_name="Authentication.CreateNonce"):
    platform: str = betterproto.string_field(1)
    appid: int = betterproto.uint32_field(2)


class CreateNonceResponse(UnifiedMessage, um_name="Authentication.CreateNonce"):
    nonce: str = betterproto.string_field(1)
    expiry: int = betterproto.uint32_field(2)


class GetTimeRemainingRequest(UnifiedMessage, um_name="Authentication.GetTimeRemaining"):
    platform: str = betterproto.string_field(1)
    appid_list: list[int] = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class TimeRemaining(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    minutes_remaining: int = betterproto.uint32_field(2)


class GetTimeRemainingResponse(UnifiedMessage, um_name="Authentication.GetTimeRemaining"):
    entries: list[TimeRemaining] = betterproto.message_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: gcsystemmsgs.proto
# plugin: python-betterproto

from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class GamePersonalDataCategoryInfo(betterproto.Message):
    type: str = betterproto.string_field(1)
    localization_token: str = betterproto.string_field(2)
    template_file: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class GetGamePersonalDataCategoriesRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class GetGamePersonalDataCategoriesResponse(betterproto.Message):
    categories: "list[GamePersonalDataCategoryInfo]" = betterproto.message_field(1)
    app_assets_basename: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class GetGamePersonalDataEntriesRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    steamid: int = betterproto.uint64_field(2)
    type: str = betterproto.string_field(3)
    continue_token: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class GetGamePersonalDataEntriesResponse(betterproto.Message):
    gceresult: int = betterproto.uint32_field(1)
    entries: list[str] = betterproto.string_field(2)
    continue_token: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class TerminateGamePersonalDataEntriesRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    steamid: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class TerminateGamePersonalDataEntriesResponse(betterproto.Message):
    gceresult: int = betterproto.uint32_field(1)



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

from __future__ import annotations

from collections import defaultdict
from dataclasses import dataclass
from typing import TYPE_CHECKING, Generic, Protocol

from typing_extensions import TypeVar

from . import utils
from .app import STEAM, PartialApp
from .enums import *
from .errors import WSException
from .models import _IOMixin
from .protobufs import UnifiedMessage, econ
from .trade import Asset, Item
from .types.id import ID32, AppID, AssetID, PublishedFileID

if TYPE_CHECKING:
    from datetime import datetime

    from ._const import ReadOnly
    from .abc import PartialUser
    from .badge import UserBadge
    from .clan import Clan, PartialClan
    from .friend import Friend
    from .protobufs import player
    from .published_file import PublishedFile
    from .state import ConnectionState
    from .types import id
    from .user import ClientUser, User

__all__ = (
    "ProfileInfo",
    "ProfileItem",
    "OwnedProfileItems",
    "EquippedProfileItems",
    "ProfileShowcaseSlot",
    "ProfileCustomisation",
    "ProfileShowcase",
    "Profile",
)


@dataclass
class ProfileInfo:
    """Represents the user's profile info."""

    created_at: datetime
    """The time at which the user was created at."""
    real_name: str | None
    """The real name of the user."""
    city_name: str | None
    """The city the user is located in."""
    state_name: str | None
    """The name of the state the user is located in."""
    country_name: str | None
    """The name of the country the user is located in."""
    headline: str | None
    """The profile's headline."""
    summary: utils.BBCodeStr
    """The user's summary."""

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} real_name={self.real_name!r}>"


@dataclass
class ProfileMovie(_IOMixin):
    __slots__ = ("url", "_state")
    _state: ConnectionState
    url: ReadOnly[str]  # TODO add more attributes like maybe created_at?


class SupportsEquip(Protocol):
    def __init__(self, *, communityitemid: int) -> None: ...


UserT = TypeVar("UserT", bound="PartialUser", default="User", covariant=True)


class ProfileItem(Generic[UserT]):
    """Represents an item on/in a user's profile."""

    __slots__ = (
        "id",
        "url",
        "name",
        "title",
        "description",
        "app",
        "type",
        "class_",
        "movie",
        "equipped_flags",
        "owner",
        "_state",
        "_um",
    )

    def __init__(
        self,
        state: ConnectionState,
        owner: UserT,
        item: player.ProfileItem,
        *,
        um: type[SupportsEquip] | None = None,
    ):
        self.id = AssetID(item.communityitemid)
        """The item's id."""
        self.url = item.image_large
        """The item's url."""
        self.name = item.name
        """The item's name."""
        self.title = item.item_title
        """The item's title."""
        self.description = item.item_description
        """The item's description."""
        self.app = PartialApp(state, id=item.appid)
        """The app the item is from."""
        self.type = ProfileItemType.try_value(item.item_type)
        """The item's type."""
        self.class_ = CommunityItemClass.try_value(item.item_class)
        """The item's class."""
        self.movie = ProfileMovie(state, item.movie_mp4)
        """The movie associated with the item."""
        self.equipped_flags = ProfileItemEquippedFlag.try_value(item.equipped_flags)
        """The item's equipped flags."""

        self.owner = owner
        self._state = state
        if um is not None:
            assert issubclass(um, UnifiedMessage)  # until we have intersections this works with pyright
        self._um = um

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} id={self.id} name={self.name!r} app={self.app!r}>"

    async def equip(self) -> None:
        """Equip the profile item."""
        if self._um is None:
            raise ValueError(f"Cannot equip {self!r}")

        msg = await self._state.ws.send_um_and_wait(self._um(communityitemid=self.id))
        if msg.result != Result.OK:
            raise WSException(msg)

    async def item(self, *, language: Language | None = None) -> Item[UserT]:
        """Resolve this to an actual item in the owner's inventory."""
        inventory = await self.owner.inventory(STEAM, language=language)
        item = utils.get(inventory, id=self.id)
        assert item
        return item


@dataclass
class OwnedProfileItems(Generic[UserT]):
    r"""Represents the :class:`ClientUser`\'s owned items."""

    backgrounds: list[ProfileItem[UserT]]
    """The backgrounds the client user owns."""
    mini_profile_backgrounds: list[ProfileItem[UserT]]
    """The mini profile backgrounds the client user owns."""
    avatar_frames: list[ProfileItem[UserT]]
    """The avatar frames the client user owns."""
    animated_avatars: list[ProfileItem[UserT]]
    """The animated avatars the client user owns."""
    modifiers: list[ProfileItem[UserT]]
    """The modifiers the client user owns."""


@dataclass
class EquippedProfileItems(Generic[UserT]):
    """Represents the items the user has equipped."""

    background: ProfileItem[UserT] | None
    """The equipped background."""
    mini_profile_background: ProfileItem[UserT] | None
    """The equipped mini profile background for the user."""
    avatar_frame: ProfileItem[UserT] | None
    """The equipped avatar frame for the user."""
    animated_avatar: ProfileItem[UserT] | None
    """The equipped animated avatar for the user."""
    modifier: ProfileItem[UserT] | None
    """The equipped modifier for the user."""


@dataclass(repr=False, slots=True)
class ProfileShowcaseSlot(Generic[UserT]):
    """Represents a showcase slot."""

    _state: ConnectionState
    owner: UserT
    name: str | None
    """The slot's name."""
    content: str | None
    """ The slot's description."""
    index: int | None
    """The slot's index."""
    app: PartialApp | None
    """The slot's associated app."""

    asset: Asset[UserT] | None
    """The :class:`Asset` the slot is associated with."""
    published_file_id: PublishedFileID | None
    """The ID of the :class:`PublishedFile` the slot is associated with."""
    badge_id: int | None
    """The ID of the :class:`UserBadge` the slot is associated with."""
    border_colour: int | None
    """The border colour of the slot."""
    clan: Clan | PartialClan | None
    """The Steam ID of the clan the slot is associated with."""
    replay_year: int | None
    """If the :attr:`ProfileShowcase.type` is :attr:`ProfileItemType.Replay` the year the replay is for"""

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} name={self.name!r} index={self.index!r} app={self.app!r}>"

    async def item(self, *, language: Language | None = None) -> Item[UserT]:
        """Fetches the associated :class:`.Item` from :attr:`asset`.

        Parameters
        ----------
        language
            The language to fetch the item in. If ``None``, the current language is used.
        """
        if self.asset is None:
            raise ValueError
        assert self.app is not None
        key = (self.asset.class_id, self.asset.instance_id)
        resp = await self._state.fetch_item_info(self.app.id, [key], language)
        return Item(self._state, self.asset.to_proto(), resp[key], self.owner)

    async def published_file(
        self, *, revision: PublishedFileRevision = PublishedFileRevision.Default, language: Language | None = None
    ) -> PublishedFile[UserT]:
        """Fetches the associated :class:`.PublishedFile` from :attr:`published_file_id`.

        Parameters
        ----------
        revision
            The revision of the published file to fetch.
        language
            The language to fetch the published file in. If ``None``, the current language is used.
        """
        if self.published_file_id is None:
            raise ValueError
        (file,) = await self._state.fetch_published_files_with_author(
            (self.published_file_id,), self.owner, revision, language
        )
        assert file
        return file

    async def badge(self) -> UserBadge[UserT]:
        """Fetches the associated :class:`.Badge` from :attr:`badge_id`."""
        if self.badge_id is None:
            raise ValueError
        badges = await self.owner.badges()
        badge = utils.get(
            badges.badges, id=self.badge_id, app__id=self.app.id if self.app and self.app.id != 753 else None
        )  # TODO manual

        assert badge
        return badge


@dataclass(repr=False, slots=True)
class ProfileShowcase(Generic[UserT]):
    """Represents a user's profile showcase."""

    _state: ConnectionState
    owner: UserT
    """The showcase's owner."""
    type: ProfileItemType
    """The showcase type."""
    large: bool
    """Whether the showcase is large."""
    active: bool
    """Whether the showcase is active."""
    purchase_id: int
    """The purchase id."""
    level: int
    """The level of the the showcase."""
    style: ProfileCustomisationStyle
    """The style of the showcase."""
    slots: list[ProfileShowcaseSlot[UserT]]
    """The slots in this showcase."""

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} type={self.type!r} active={self.active!r} level={self.level!r} style={self.style!r}>"

    async def items(self, *, language: Language | None = None) -> list[Item[UserT]]:
        """Fetches the associated :class:`.Item`\\s for the entire showcase.

        Parameters
        ----------
        language
            The language to fetch the items in. If ``None``, the current language is used.
        """
        asset_map: defaultdict[AppID, dict[id.CacheKey, Asset[UserT]]] = defaultdict(dict)

        for slot in self.slots:
            if slot.asset:
                asset_map[slot.asset._app_id][(slot.asset.class_id, slot.asset.instance_id)] = slot.asset

        return [
            Item(self._state, assets[key].to_proto(), description, self.owner)
            for app_id, assets in asset_map.items()
            for key, description in (await self._state.fetch_item_info(app_id, assets, language)).items()
        ]

    async def published_files(
        self, *, revision: PublishedFileRevision = PublishedFileRevision.Default, language: Language | None = None
    ) -> list[PublishedFile[UserT]]:
        """Fetches the associated :class:`.PublishedFile` for the entire showcase.

        Parameters
        ----------
        revision
            The revision of the file to fetch.
        language
            The language to fetch the file in. If ``None``, the current language is used.
        """
        published_files = await self._state.fetch_published_files_with_author(
            (slot.published_file_id for slot in self.slots if slot.published_file_id), self.owner, revision, language
        )
        assert all(published_files)
        return published_files  # type: ignore  # needs HKT to be fixed

    async def badges(self) -> list[UserBadge[UserT]]:
        """Fetches the associated :class:`.Badge`\\s for the entire showcase."""
        all_badges = await self.owner.badges()
        badges = [
            utils.get(all_badges, id=slot.badge_id, app__id=slot.app.id)
            for slot in self.slots
            if slot.badge_id and slot.app
        ]
        assert all(badges)
        return badges  # type: ignore


@dataclass(slots=True)
class ProfileTheme:
    id: str
    name: str


@dataclass(slots=True)
class PurchasedCustomisation:  # TODO should this be merged into ProfileCustomisationSlot.purchase_id if the ids match
    id: int
    type: ProfileCustomisationStyle
    level: int


class ProfileCustomisation(Generic[UserT]):
    """Represents a user's profile customisations."""

    __slots__ = (
        "showcases",
        "slots_available",
        "profile_theme",
        "purchased_customisations",
        "owner",
        "_state",
    )

    def __init__(self, state: ConnectionState, user: UserT, proto: player.GetProfileCustomizationResponse):
        from .clan import PartialClan

        self.showcases = [
            ProfileShowcase(
                state,
                owner=user,
                slots=[
                    ProfileShowcaseSlot(
                        state,
                        owner=user,
                        index=slot.slot,
                        app=PartialApp(state, id=slot.appid) if slot.appid else None,
                        published_file_id=PublishedFileID(slot.publishedfileid) or None,
                        name=slot.title or None,
                        content=slot.notes or None,
                        badge_id=slot.badgeid or None,
                        border_colour=slot.border_color or None,
                        asset=(
                            Asset(
                                state,
                                econ.Asset(
                                    assetid=slot.item_assetid,
                                    appid=slot.appid,
                                    contextid=slot.item_contextid,
                                    instanceid=slot.item_instanceid,
                                    classid=slot.item_classid,
                                    amount=1,
                                ),
                                user,
                            )
                            if slot.item_assetid
                            else None
                        ),
                        clan=(
                            (state.get_clan(ID32(slot.accountid)) or PartialClan(state, slot.accountid))
                            if slot.accountid
                            else None
                        ),
                        replay_year=slot.replay_year or None,
                    )
                    for slot in customisation.slots
                ],
                type=ProfileItemType.try_value(customisation.customization_type),
                large=customisation.large,
                active=customisation.active,
                style=ProfileCustomisationStyle.try_value(customisation.customization_style),
                purchase_id=customisation.purchaseid,
                level=customisation.level,
            )
            for customisation in proto.customizations
        ]
        self.slots_available = proto.slots_available
        self.profile_theme = ProfileTheme(proto.profile_theme.theme_id, proto.profile_theme.title)
        self.purchased_customisations = [
            PurchasedCustomisation(
                customisation.purchaseid,
                ProfileCustomisationStyle.try_value(customisation.customization_type),
                customisation.level,
            )
            for customisation in proto.purchased_customizations
        ]
        self.owner = user
        self._state = state

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} owner={self.owner!r}>"


class Profile(EquippedProfileItems[UserT], ProfileCustomisation[UserT]):
    """Represents a user's complete profile."""

    __slots__ = ()

    def __init__(self, equipped_items: EquippedProfileItems[UserT], customisation_info: ProfileCustomisation[UserT]):
        utils.update_class(equipped_items, self)
        utils.update_class(customisation_info, self)

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__} owner={self.owner!r}>"


class FriendProfile(ProfileInfo, Profile["Friend"]):
    """Represents a friend's complete profile."""

    def __init__(
        self, equipped_items: EquippedProfileItems, info: ProfileInfo, customisation_info: ProfileCustomisation
    ):
        utils.update_class(equipped_items, self)
        utils.update_class(info, self)
        utils.update_class(customisation_info, self)


class ClientUserProfile(ProfileInfo, Profile["ClientUser"]):
    """Represents a :class:`ClientUser`'s full profile."""

    def __init__(
        self,
        equipped_items: EquippedProfileItems[ClientUser],
        info: ProfileInfo,
        customisation_info: ProfileCustomisation[ClientUser],
        items: OwnedProfileItems[ClientUser],
    ):
        utils.update_class(equipped_items, self)
        utils.update_class(info, self)
        utils.update_class(customisation_info, self)

        self.owned_backgrounds = items.backgrounds
        """The backgrounds the client user owns."""
        self.owned_mini_profile_backgrounds = items.mini_profile_backgrounds
        """The mini profile backgrounds the client user owns."""
        self.owned_avatar_frames = items.avatar_frames
        """The avatar frames the client user owns."""
        self.owned_animated_avatars = items.animated_avatars
        """The animated avatars the client user owns."""
        self.owned_modifiers = items.modifiers
        """The modifiers the client user owns."""



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any, Generic

from typing_extensions import TypeVar

from ..._const import _HasChatGroupMixin
from ...abc import Message, Messageable, PartialUser

if TYPE_CHECKING:
    from collections.abc import AsyncGenerator, Coroutine
    from datetime import datetime
    from inspect import Parameter

    from ...chat import Member, PartialMember
    from ...friend import Friend
    from ...media import Media
    from ...user import ClientUser, User
    from .bot import Bot
    from .commands import Command
    from .utils import Shlex


__all__ = ("Context",)

BotT = TypeVar("BotT", bound="Bot", default="Bot", covariant=True)


class Context(Generic[BotT], Messageable["Message"], _HasChatGroupMixin):
    """Represents the context of a command.

    Attributes
    ----------
    message
        The message the context was generated from.
    prefix
        The prefix of the message the context was generated from
    author
        The author of the message.
    channel
        The channel the message was sent in.
    clan
        The clan the message was sent in ``None`` if the message wasn't sent in a clan.
    group
        The group the message was sent in ``None`` if the message wasn't sent in a group.
    bot
        The bot instance.
    command
        The command the context is attached to.
    cog
        The cog the command is in.
    """

    def __init__(
        self,
        bot: BotT,
        message: Message[User | ClientUser | Friend | Member | PartialMember],
        lex: Shlex,
        prefix: str | None,
        command: Command | None = None,
        invoked_with: str | None = None,
    ):
        self.bot = bot
        self.message = message
        self.lex = lex
        self.prefix = prefix

        self.command = command
        self.cog = self.command.cog if self.command is not None else None
        self.invoked_with = invoked_with

        self.author = self.message.author
        self.channel = self.message.channel
        self.clan = self.message.clan
        self.group = self.message.group
        self._state = self.message._state

        self.args: tuple[Any, ...] | None = None
        self.kwargs: dict[str, Any] | None = None
        self.command_failed: bool = False
        self.current_param: Parameter | None = None

    def _send_message(self, content: str) -> Coroutine[Any, Any, Message]:
        return self.channel._send_message(content)

    def _send_media(self, media: Media) -> Coroutine[Any, Any, None]:
        return self.channel._send_media(media)

    async def invoke(self) -> None:
        """A shortcut method that invokes the current context using :meth:`~steam.ext.commands.Command.invoke`.

        Equivalent to

        .. code:: python

            await ctx.command.invoke(ctx)
        """
        if self.command is None:
            return
        await self.command.invoke(self)

    def history(
        self,
        *,
        limit: int | None = 100,
        before: datetime | None = None,
        after: datetime | None = None,
    ) -> AsyncGenerator[Message[PartialUser], None]:
        """A shortcut method that gets the current channel's history.

        Equivalent to

        .. code:: python

            ctx.channel.history(**kwargs)
        """
        return self.channel.history(limit=limit, before=before, after=after)

    @property
    def valid(self) -> bool:
        """Whether or not the context could be invoked."""
        return self.prefix is not None and self.command is not None



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

from __future__ import annotations

from collections import deque
from collections.abc import Coroutine
from typing import TYPE_CHECKING, Any, TypeAlias, TypeVar, overload

from .errors import MissingClosingQuotation

if TYPE_CHECKING:
    from collections.abc import Generator

_T = TypeVar("_T")
_VT = TypeVar("_VT")

Coro: TypeAlias = Coroutine[Any, Any, _T]


class CaseInsensitiveDict(dict[str, _VT]):
    """A dictionary where keys are case insensitive."""

    def __init__(self, **kwargs: _VT):
        super().__init__(**{k.lower(): v for k, v in kwargs.items()})

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}({', '.join(f'{k}={v!r}' for k, v in self.items())})"

    def __contains__(self, k: object, /) -> bool:
        if not isinstance(k, str):
            return False
        return super().__contains__(k.lower())

    def __delitem__(self, k: str, /) -> None:
        super().__delitem__(k.lower())

    def __getitem__(self, k: str, /) -> _VT:
        return super().__getitem__(k.lower())

    def __setitem__(self, k: str, v: _VT, /) -> None:
        super().__setitem__(k.lower(), v)

    @overload
    def get(self, k: str, /) -> _VT | None: ...

    @overload
    def get(self, k: str, /, default: _T | None = None) -> _VT | _T | None: ...

    def get(self, k: str, /, default: _T | None = None) -> _VT | _T | None:
        return super().get(k.lower(), default)

    @overload
    def pop(self, k: str, /) -> _VT | None: ...

    @overload
    def pop(self, k: str, /, default: _T | None = None) -> _VT | _T | None: ...

    def pop(self, k: str, /, default: _T | None = None) -> _VT | _T | None:
        return super().pop(k.lower())


def _end_of_quote_finder(in_stream: str, location: int) -> int:
    end_of_quote_index = in_stream.find('"', location)
    if end_of_quote_index == -1:
        raise MissingClosingQuotation(location)
    if in_stream[end_of_quote_index - 1] == "\\":  # quote is escaped carry on searching
        return _end_of_quote_finder(in_stream, end_of_quote_index + 1)
    return end_of_quote_index


def remove_quotes(string: str) -> str:
    """
    >>> remove_quotes('"only a quote at the start')
    '"quoted only at the start'
    >>> remove_quotes('"quoted all the way"')
    'quoted all the way'
    """
    return string[1:-1] if (string[-1:], string[:1]) == ('"', '"') else string


class Shlex:
    """A simple lexical analyser.
    This should be a simpler and faster version of :class:`shlex.shlex` in posix mode.
    """

    __slots__ = ("in_stream", "position", "end", "_undo_pushback")

    def __init__(self, in_stream: str):
        self.in_stream = in_stream.replace("‘", '"').replace("’", '"').replace("“", '"').replace("”", '"').strip()
        self.position = 0
        self.end = len(self.in_stream)
        self._undo_pushback: deque[int] = deque()

    def read(self) -> str | None:
        if self.position >= self.end:
            return None

        while True:
            start = self.position
            characters: list[str] = []

            for character in self.in_stream[self.position :]:
                self.position += 1
                if character.isspace():
                    break

                if character == '"':
                    before = self.position
                    if self.in_stream[before - 2] != "\\":  # quote is escaped carry on searching
                        end_of_quote = _end_of_quote_finder(self.in_stream, before)
                        self.position = end_of_quote + 2
                        self._undo_pushback.append(start)
                        return remove_quotes(self.in_stream[before:end_of_quote])

                    characters.pop()
                characters.append(character)

            if ret := remove_quotes("".join(characters)):
                self._undo_pushback.append(start)
                return ret

    def undo(self) -> None:
        try:
            self.position = self._undo_pushback.pop()
        except IndexError:
            pass

    @property
    def rest(self) -> str:
        return self.in_stream[self.position :]

    def __repr__(self) -> str:
        attrs = ("in_stream", "position", "end")
        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
        return f"<{self.__class__.__class__} {' '.join(resolved)}>"

    def __iter__(self) -> Generator[str, None, None]:
        while (token := self.read()) is not None:
            yield token



"""Licensed under The MIT License (MIT) - Copyright (c) 2020-present James H-B. See LICENSE"""

from __future__ import annotations

import asyncio
import time
from typing import TYPE_CHECKING, cast

from typing_extensions import Self

from ...chat import PartialMember
from ...enums import IntEnum
from .errors import CommandOnCooldown

if TYPE_CHECKING:
    from .context import Context

__all__ = (
    "BucketType",
    "Cooldown",
)


class BucketTypeType:
    def __new__(cls, *values: int | None) -> Self:
        return cast(Self, values)


class BucketType(IntEnum):
    """A enumeration that handles cooldowns.

    Each attribute operates on a per x basis. So :attr:`User` operates on a per :class:`~steam.User` basis.
    """

    # fmt: off
    Default   = 0
    """The default :class:`BucketType` this operates on a global basis."""
    User      = 1
    """The :class:`BucketType` for a :class:`steam.User`."""
    Member    = 2
    """The :class:`BucketType` for a :class:`steam.User` and a :class:`steam.Clan`."""
    Role      = 5
    """The :class:`BucketType` for a :class:`steam.Role`."""
    Channel   = 6
    """The :class:`BucketType` for a :class:`steam.Channel`."""
    Admin     = 7
    """The :class:`BucketType` for a :class:`steam.Clan`'s :attr:`steam.Clan.admins`."""
    ChatGroup = 8
    """The :class:`BucketType` for a :class:`steam.ChatGroup`."""
    # fmt: on

    def get_bucket(self, ctx: Context) -> BucketTypeType:
        """Get a bucket key for a message or context.

        Parameters
        ----------
        ctx
            The message or context to get the bucket for.
        """
        match self:
            case BucketType.Default:
                return BucketTypeType(0)
            case BucketType.User:
                return BucketTypeType(ctx.author.id)
            case BucketType.Member:
                if ctx.clan or ctx.group:
                    return BucketTypeType(ctx._chat_group.id64, ctx.author.id)
                return BucketTypeType(0, ctx.author.id)
            case BucketType.Role:
                assert isinstance(ctx.author, PartialMember)
                return BucketTypeType((ctx.clan and ctx.author.top_role and ctx.author.top_role.id), ctx.author.id)
            case BucketType.Channel:
                if hasattr(ctx.channel, "id"):
                    return BucketTypeType(ctx.channel.id)  # type: ignore
                return BucketTypeType(ctx.author.id)
            case BucketType.Admin:
                return BucketTypeType((ctx.clan and (ctx.author in ctx.clan.officers)), ctx.author.id)
            case BucketType.ChatGroup:
                return BucketTypeType((ctx.clan or ctx.group or ctx.author).id)


class Cooldown:
    """The class that holds a command's cooldown."""

    bucket: BucketType
    """The bucket that should be used to determine this command's cooldown."""

    def __init__(self, rate: int, per: float, bucket: BucketType):
        self._rate = rate
        self._per = per
        self.bucket = bucket
        self.reset()

    def reset(self) -> None:
        """Reset the command's cooldown."""
        self._queue: dict[float, BucketTypeType] = {}

    def _calls(self, bucket: BucketTypeType) -> list[tuple[float, BucketTypeType]]:
        return [(t, b) for t, b in self._queue.items() if b == bucket]

    async def expire_cache(self, bucket: BucketTypeType, now: float) -> None:
        self._queue[now] = bucket
        await asyncio.sleep(self._per)
        self._queue.pop(now, None)

    def get_retry_after(self, bucket: BucketTypeType, now: float) -> float:
        """Get the retry after for a command.

        Parameters
        ----------
        bucket
            The bucket to find in the cache.
        now
            The UNIX timestamp to find times after.
        """
        calls = self._calls(bucket)
        if not calls:
            return 0.0
        last_call = calls[0][0]
        if last_call + self._per >= now and len(calls) >= self._rate:
            return last_call + self._per - now
        return 0.0

    def __call__(self, ctx: Context) -> None:
        """Invoke the command's cooldown properly and raise if the command is on cooldown.

        Parameters
        ----------
        ctx
            The context for invocation to check for a cooldown on.
        """
        bucket = self.bucket.get_bucket(ctx)
        now = time.time()
        ctx._state._tg.create_task(self.expire_cache(bucket, now))
        if retry_after := self.get_retry_after(bucket, now):
            raise CommandOnCooldown(retry_after)
